from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.arm.base_resource_check import BaseResourceCheck

# https://docs.microsoft.com/en-us/azure/templates/microsoft.sql/2019-06-01-preview/servers
# https://docs.microsoft.com/en-us/azure/templates/microsoft.sql/2017-03-01-preview/servers/databases/auditingsettings


class SQLServerAuditingEnabled(BaseResourceCheck):
    # this should be a graph check, due to the possible connection between
    # Microsoft.Sql/servers -> Microsoft.Sql/servers/auditingSettings
    # Microsoft.Sql/servers -> Microsoft.Sql/servers/databases/auditingSettings

    def __init__(self) -> None:
        name = "Ensure that 'Auditing' is set to 'Enabled' for SQL servers"
        id = "CKV_AZURE_23"
        supported_resources = ("Microsoft.Sql/servers", "Microsoft.Sql/servers/databases")
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        resources = conf.get("resources")
        if resources and isinstance(resources, list):
            for resource in resources:
                if resource.get("type") in (
                    "auditingSettings",
                    "Microsoft.Sql/servers/auditingSettings",
                    "Microsoft.Sql/servers/databases/auditingSettings",
                ):
                    properties = resource.get("properties")
                    if properties:
                        state = properties.get("state")
                        if state and state.lower() == "enabled":
                            return CheckResult.PASSED

        return CheckResult.FAILED


check = SQLServerAuditingEnabled()



from __future__ import annotations
from typing import Any

from checkov.circleci_pipelines.base_circleci_pipelines_check import BaseCircleCIPipelinesCheck
from checkov.common.models.enums import CheckResult
from checkov.yaml_doc.enums import BlockType


class DetectImageUsage(BaseCircleCIPipelinesCheck):
    def __init__(self) -> None:
        name = "Detecting image usages in circleci pipelines"
        id = "CKV_CIRCLECIPIPELINES_8"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=(
                "executors.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}",
            )
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        return CheckResult.PASSED, conf


check = DetectImageUsage()



import json
import logging
from abc import abstractmethod
from typing import Dict, List, Any, Union

from cloudsplaining.scan.policy_document import PolicyDocument

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.data.base_check import BaseDataCheck
from checkov.terraform.checks.utils.iam_terraform_document_to_policy_converter import (
    convert_terraform_conf_to_iam_policy,
)


class BaseCloudsplainingIAMCheck(BaseDataCheck):
    # creating a PolicyDocument is computational expensive,
    # therefore a cache is defined at class level
    policy_document_cache: Dict[str, PolicyDocument] = {}  # noqa: CCE003

    def __init__(self, name: str, id: str) -> None:
        super().__init__(name=name, id=id, categories=[CheckCategories.IAM], supported_data=["aws_iam_policy_document"])

    def scan_data_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        if "statement" in conf.keys():
            try:
                if self.entity_path not in BaseCloudsplainingIAMCheck.policy_document_cache.keys():
                    converted_conf = convert_terraform_conf_to_iam_policy(conf)
                    policy = PolicyDocument(converted_conf)
                    BaseCloudsplainingIAMCheck.policy_document_cache[self.entity_path] = policy
                violations = self.cloudsplaining_analysis(
                    BaseCloudsplainingIAMCheck.policy_document_cache[self.entity_path]
                )
            except Exception:
                # this might occur with templated iam policies where ARN is not in place or similar
                logging.debug(f"could not run cloudsplaining analysis on policy {conf}")
                return CheckResult.UNKNOWN
            if violations:
                logging.debug(f"detailed cloudsplainging finding: {json.dumps(violations, indent=2, default=str)}")
                return CheckResult.FAILED
        return CheckResult.PASSED

    @abstractmethod
    def cloudsplaining_analysis(self, policy: PolicyDocument) -> Union[List[str], List[Dict[str, Any]]]:
        raise NotImplementedError()



from typing import Optional, Any, Dict, Collection
from collections.abc import Iterable
from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver


class IntersectsAttributeSolver(BaseAttributeSolver):
    operator = Operators.INTERSECTS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None

        if isinstance(self.value, str) and isinstance(attr, Iterable):
            return self.value in attr

        if isinstance(self.value, Collection) and isinstance(attr, Iterable):
            return any(i in self.value for i in attr)

        return False



from __future__ import annotations

from checkov.github.repository_creation_is_limited import RepositoryCreationIsLimited


class GithubInternalRepositoryCreationIsLimited(RepositoryCreationIsLimited):
    def __init__(self) -> None:
        name = "Ensure internal repository creation is limited to specific members"
        id = "CKV_GITHUB_23"
        super().__init__(id=id, name=name)

    def get_evaluated_keys(self) -> list[str]:
        return ["members_can_create_internal_repositories"]


check = GithubInternalRepositoryCreationIsLimited()



from __future__ import annotations

import logging
import fnmatch
from collections.abc import Iterable
from typing import Any, Set, Optional, Union, List, TYPE_CHECKING, Dict

from checkov.common.bridgecrew.code_categories import CodeCategoryMapping, CodeCategoryConfiguration
from checkov.common.bridgecrew.severities import Severity, Severities
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.type_forcers import convert_csv_string_arg_to_list

if TYPE_CHECKING:
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.graph.checks_infra.base_check import BaseGraphCheck


class RunnerFilter(object):
    # NOTE: This needs to be static because different filters may be used at load time versus runtime
    #       (see note in BaseCheckRegistery.register). The concept of which checks are external is
    #       logically a "static" concept anyway, so this makes logical sense.
    __EXTERNAL_CHECK_IDS: Set[str] = set()

    def __init__(
            self,
            framework: Optional[List[str]] = None,
            checks: Union[str, List[str], None] = None,
            skip_checks: Union[str, List[str], None] = None,
            include_all_checkov_policies: bool = True,
            download_external_modules: bool = False,
            external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
            evaluate_variables: bool = True,
            runners: Optional[List[str]] = None,
            skip_framework: Optional[List[str]] = None,
            excluded_paths: Optional[List[str]] = None,
            all_external: bool = False,
            var_files: Optional[List[str]] = None,
            skip_cve_package: Optional[List[str]] = None,
            use_enforcement_rules: bool = False,
            filtered_policy_ids: Optional[List[str]] = None,
            show_progress_bar: Optional[bool] = True,
            run_image_referencer: bool = False,
            enable_secret_scan_all_files: bool = False,
            block_list_secret_scan: Optional[List[str]] = None,
            deep_analysis: bool = False,
            repo_root_for_plan_enrichment: Optional[List[str]] = None
    ) -> None:

        checks = convert_csv_string_arg_to_list(checks)
        skip_checks = convert_csv_string_arg_to_list(skip_checks)

        self.use_enforcement_rules = use_enforcement_rules
        self.enforcement_rule_configs: Optional[Dict[str, Severity]] = None

        # we will store the lowest value severity we find in checks, and the highest value we find in skip-checks
        # so the logic is "run all checks >= severity" and/or "skip all checks <= severity"
        self.check_threshold = None
        self.skip_check_threshold = None
        self.checks = []
        self.skip_checks = []
        self.show_progress_bar = show_progress_bar

        # split out check/skip thresholds so we can access them easily later
        for val in (checks or []):
            if val.upper() in Severities:
                val = val.upper()
                if not self.check_threshold or self.check_threshold.level > Severities[val].level:
                    self.check_threshold = Severities[val]
            else:
                self.checks.append(val)

        for val in (skip_checks or []):
            if val.upper() in Severities:
                val = val.upper()
                if not self.skip_check_threshold or self.skip_check_threshold.level < Severities[val].level:
                    self.skip_check_threshold = Severities[val]
            else:
                self.skip_checks.append(val)

        self.include_all_checkov_policies = include_all_checkov_policies

        self.framework: "Iterable[str]" = framework if framework else ["all"]
        if skip_framework:
            if "all" in self.framework:
                if runners is None:
                    runners = []

                self.framework = set(runners) - set(skip_framework)
            else:
                self.framework = set(self.framework) - set(skip_framework)
        logging.debug(f"Resultant set of frameworks (removing skipped frameworks): {','.join(self.framework)}")

        self.download_external_modules = download_external_modules
        self.external_modules_download_path = external_modules_download_path
        self.evaluate_variables = evaluate_variables
        self.excluded_paths = excluded_paths or []
        self.all_external = all_external
        self.var_files = var_files
        self.skip_cve_package = skip_cve_package
        self.filtered_policy_ids = filtered_policy_ids or []
        self.run_image_referencer = run_image_referencer
        self.enable_secret_scan_all_files = enable_secret_scan_all_files
        self.block_list_secret_scan = block_list_secret_scan
        self.suppressed_policies: List[str] = []
        self.deep_analysis = deep_analysis
        self.repo_root_for_plan_enrichment = repo_root_for_plan_enrichment

    def apply_enforcement_rules(self, enforcement_rule_configs: Dict[str, CodeCategoryConfiguration]) -> None:
        self.enforcement_rule_configs = {}
        for report_type, code_category in CodeCategoryMapping.items():
            config = enforcement_rule_configs.get(code_category)
            if not config:
                raise Exception(f'Could not find an enforcement rule config for category {code_category} (runner: {report_type})')
            self.enforcement_rule_configs[report_type] = config.soft_fail_threshold

    def should_run_check(
        self,
        check: BaseCheck | BaseGraphCheck | None = None,
        check_id: str | None = None,
        bc_check_id: str | None = None,
        severity: Severity | None = None,
        report_type: str | None = None
    ) -> bool:
        if check:
            check_id = check.id
            bc_check_id = check.bc_id
            severity = check.severity

        assert check_id is not None  # nosec (for mypy (and then for bandit))

        # apply enforcement rules if specified, but let --check/--skip-check with a severity take priority
        if self.use_enforcement_rules and report_type:
            if not self.check_threshold and not self.skip_check_threshold:
                check_threshold = self.enforcement_rule_configs[report_type]  # type:ignore[index] # mypy thinks it might be null
                skip_check_threshold = None
            else:
                check_threshold = self.check_threshold
                skip_check_threshold = self.skip_check_threshold
        else:
            if self.use_enforcement_rules:
                # this is a warning for us (but there is nothing the user can do about it)
                logging.debug(f'Use enforcement rules is true, but check {check_id} was not passed to the runner filter with a report type')
            check_threshold = self.check_threshold
            skip_check_threshold = self.skip_check_threshold

        run_severity = severity and check_threshold and severity.level >= check_threshold.level
        explicit_run = self.checks and self.check_matches(check_id, bc_check_id, self.checks)
        implicit_run = not self.checks and not check_threshold
        is_external = RunnerFilter.is_external_check(check_id)
        is_policy_filtered = self.is_policy_filtered(check_id)
        # True if this check is present in the allow list, or if there is no allow list
        # this is not necessarily the return value (need to apply other filters)
        should_run_check = (
            run_severity or
            explicit_run or
            implicit_run or
            (is_external and self.all_external)
        )

        if not should_run_check:
            return False

        # If a policy is not present in the list of filtered policies, it should not be run - implicitly or explicitly.
        # It can, however, be skipped.
        if not is_policy_filtered:
            should_run_check = False

        skip_severity = severity and skip_check_threshold and severity.level <= skip_check_threshold.level
        explicit_skip = self.skip_checks and self.check_matches(check_id, bc_check_id, self.skip_checks)

        should_skip_check = (
            skip_severity or
            explicit_skip or
            (not bc_check_id and not self.include_all_checkov_policies and not is_external and not explicit_run) or
            check_id in self.suppressed_policies
        )

        if should_skip_check:
            result = False
        elif should_run_check:
            result = True
        else:
            result = False

        logging.debug(f'Should run check {check_id}: {result}')
        return result

    @staticmethod
    def check_matches(check_id: str,
                      bc_check_id: Optional[str],
                      pattern_list: List[str]) -> bool:
        return any(
            (fnmatch.fnmatch(check_id, pattern) or (bc_check_id and fnmatch.fnmatch(bc_check_id, pattern))) for pattern
            in pattern_list)

    def within_threshold(self, severity: Severity) -> bool:
        above_min = (not self.check_threshold) or self.check_threshold.level <= severity.level
        below_max = self.skip_check_threshold and self.skip_check_threshold.level >= severity.level
        return above_min and not below_max

    @staticmethod
    def notify_external_check(check_id: str) -> None:
        RunnerFilter.__EXTERNAL_CHECK_IDS.add(check_id)

    @staticmethod
    def is_external_check(check_id: str) -> bool:
        return check_id in RunnerFilter.__EXTERNAL_CHECK_IDS

    def is_policy_filtered(self, check_id: str) -> bool:
        if not self.filtered_policy_ids:
            return True
        return check_id in self.filtered_policy_ids
    
    def to_dict(self) -> Dict[str, Any]:
        result: Dict[str, Any] = {}
        for key, value in self.__dict__.items():
            result[key] = value
        return result

    @staticmethod
    def from_dict(obj: Dict[str, Any]) -> RunnerFilter:
        framework = obj.get('framework')
        checks = obj.get('checks')
        skip_checks = obj.get('skip_checks')
        include_all_checkov_policies = obj.get('include_all_checkov_policies')
        if include_all_checkov_policies is None:
            include_all_checkov_policies = True
        download_external_modules = obj.get('download_external_modules')
        if download_external_modules is None:
            download_external_modules = False
        external_modules_download_path = obj.get('external_modules_download_path')
        if external_modules_download_path is None:
            external_modules_download_path = DEFAULT_EXTERNAL_MODULES_DIR
        evaluate_variables = obj.get('evaluate_variables')
        if evaluate_variables is None:
            evaluate_variables = True
        runners = obj.get('runners')
        skip_framework = obj.get('skip_framework')
        excluded_paths = obj.get('excluded_paths')
        all_external = obj.get('all_external')
        if all_external is None:
            all_external = False
        var_files = obj.get('var_files')
        skip_cve_package = obj.get('skip_cve_package')
        use_enforcement_rules = obj.get('use_enforcement_rules')
        if use_enforcement_rules is None:
            use_enforcement_rules = False
        filtered_policy_ids = obj.get('filtered_policy_ids')
        show_progress_bar = obj.get('show_progress_bar')
        if show_progress_bar is None:
            show_progress_bar = True
        run_image_referencer = obj.get('run_image_referencer')
        if run_image_referencer is None:
            run_image_referencer = False
        enable_secret_scan_all_files = bool(obj.get('enable_secret_scan_all_files'))
        block_list_secret_scan = obj.get('block_list_secret_scan')
        runner_filter = RunnerFilter(framework, checks, skip_checks, include_all_checkov_policies,
                                     download_external_modules, external_modules_download_path, evaluate_variables,
                                     runners, skip_framework, excluded_paths, all_external, var_files,
                                     skip_cve_package, use_enforcement_rules, filtered_policy_ids, show_progress_bar,
                                     run_image_referencer, enable_secret_scan_all_files, block_list_secret_scan)
        return runner_filter

    def set_suppressed_policies(self, policy_level_suppressions: List[str]) -> None:
        logging.debug(f"Received the following policy-level suppressions, that will be skipped from running: {policy_level_suppressions}")
        self.suppressed_policies = policy_level_suppressions



from __future__ import annotations

import datetime
import linecache
import logging
import os
import re
from pathlib import Path
from typing import TYPE_CHECKING, cast

from detect_secrets import SecretsCollection
from detect_secrets.core import scan
from detect_secrets.settings import transient_settings


from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    integration as metadata_integration
from checkov.common.bridgecrew.severities import Severity
from checkov.common.comment.enum import COMMENT_REGEX
from checkov.common.models.consts import SUPPORTED_FILE_EXTENSIONS
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
from checkov.common.runners.base_runner import ignored_directories
from checkov.common.typing import _CheckResult
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.dockerfile import is_docker_file
from checkov.common.util.secrets import omit_secret_value_from_line
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from checkov.common.util.tqdm_utils import ProgressBar
    from detect_secrets.core.potential_secret import PotentialSecret

SOURCE_CODE_EXTENSION = ['.py', '.js', '.properties', '.pem', '.php', '.xml', '.ts', '.env', '.java', '.rb',
                         'go', 'cs', '.txt']
SECRET_TYPE_TO_ID = {
    'Artifactory Credentials': 'CKV_SECRET_1',
    'AWS Access Key': 'CKV_SECRET_2',
    'Azure Storage Account access key': 'CKV_SECRET_3',
    'Basic Auth Credentials': 'CKV_SECRET_4',
    'Cloudant Credentials': 'CKV_SECRET_5',
    'Base64 High Entropy String': 'CKV_SECRET_6',
    'IBM Cloud IAM Key': 'CKV_SECRET_7',
    'IBM COS HMAC Credentials': 'CKV_SECRET_8',
    'JSON Web Token': 'CKV_SECRET_9',
    'Secret Keyword': 'CKV_SECRET_10',
    'Mailchimp Access Key': 'CKV_SECRET_11',
    'NPM tokens': 'CKV_SECRET_12',
    'Private Key': 'CKV_SECRET_13',
    'Slack Token': 'CKV_SECRET_14',
    'SoftLayer Credentials': 'CKV_SECRET_15',
    'Square OAuth Secret': 'CKV_SECRET_16',
    'Stripe Access Key': 'CKV_SECRET_17',
    'Twilio API Key': 'CKV_SECRET_18',
    'Hex High Entropy String': 'CKV_SECRET_19'
}
CHECK_ID_TO_SECRET_TYPE = {v: k for k, v in SECRET_TYPE_TO_ID.items()}

PROHIBITED_FILES = ['Pipfile.lock', 'yarn.lock', 'package-lock.json', 'requirements.txt']

MAX_FILE_SIZE = int(os.getenv('CHECKOV_MAX_FILE_SIZE', '5000000'))  # 5 MB is default limit


class Runner(BaseRunner[None]):
    check_type = CheckType.SECRETS  # noqa: CCE003  # a static attribute

    def run(
            self,
            root_folder: str | None,
            external_checks_dir: list[str] | None = None,
            files: list[str] | None = None,
            runner_filter: RunnerFilter | None = None,
            collect_skip_comments: bool = True
    ) -> Report:
        runner_filter = runner_filter or RunnerFilter()
        current_dir = Path(__file__).parent
        secrets = SecretsCollection()
        plugins_used = [
            {'name': 'AWSKeyDetector'},
            {'name': 'ArtifactoryDetector'},
            {'name': 'AzureStorageKeyDetector'},
            {'name': 'BasicAuthDetector'},
            {'name': 'CloudantDetector'},
            {'name': 'IbmCloudIamDetector'},
            {'name': 'MailchimpDetector'},
            {'name': 'PrivateKeyDetector'},
            {'name': 'SlackDetector'},
            {'name': 'SoftlayerDetector'},
            {'name': 'SquareOAuthDetector'},
            {'name': 'StripeDetector'},
            {'name': 'TwilioKeyDetector'},
            {'name': 'EntropyKeywordCombinator', 'path': f'file://{current_dir}/plugins/entropy_keyword_combinator.py'}
        ]

        detector_path = f"{current_dir}/plugins/custom_regex_detector.py"
        logging.info(f"Custom detector found at {detector_path}. Loading...")
        plugins_used.append({
            'name': 'CustomRegexDetector',
            'path': f'file://{detector_path}'
        })
        with transient_settings({
            # Only run scans with only these plugins.
            'plugins_used': plugins_used
        }) as settings:
            report = Report(self.check_type)
            if not runner_filter.show_progress_bar:
                self.pbar.turn_off_progress_bar()

            # Implement non IaC files (including .terraform dir)
            files_to_scan = files or []
            excluded_paths = (runner_filter.excluded_paths or []) + ignored_directories + [DEFAULT_EXTERNAL_MODULES_DIR]
            if root_folder:
                enable_secret_scan_all_files = runner_filter.enable_secret_scan_all_files
                block_list_secret_scan = runner_filter.block_list_secret_scan or []
                block_list_secret_scan_lower = [file_type.lower() for file_type in block_list_secret_scan]
                for root, d_names, f_names in os.walk(root_folder):
                    filter_ignored_paths(root, d_names, excluded_paths)
                    filter_ignored_paths(root, f_names, excluded_paths)
                    for file in f_names:
                        if enable_secret_scan_all_files:
                            if is_docker_file(file):
                                if 'dockerfile' not in block_list_secret_scan_lower:
                                    files_to_scan.append(os.path.join(root, file))
                            elif f".{file.split('.')[-1]}" not in block_list_secret_scan_lower:
                                files_to_scan.append(os.path.join(root, file))
                        elif file not in PROHIBITED_FILES and f".{file.split('.')[-1]}" in SUPPORTED_FILE_EXTENSIONS or is_docker_file(file):
                            files_to_scan.append(os.path.join(root, file))
            logging.info(f'Secrets scanning will scan {len(files_to_scan)} files')

            settings.disable_filters(*['detect_secrets.filters.heuristic.is_indirect_reference'])

            self.pbar.initiate(len(files_to_scan))
            self._scan_files(files_to_scan, secrets, self.pbar)
            self.pbar.close()
            secrets_duplication: dict[str, bool] = {}
            for _, secret in secrets:
                check_id = getattr(secret, "check_id", SECRET_TYPE_TO_ID.get(secret.type))
                if not check_id:
                    continue
                secret_key = f'{secret.filename}_{secret.line_number}_{secret.secret_hash}'
                if secret_key in secrets_duplication:
                    continue
                else:
                    secrets_duplication[secret_key] = True
                bc_check_id = metadata_integration.get_bc_id(check_id)
                severity = metadata_integration.get_severity(check_id)
                if not runner_filter.should_run_check(check_id=check_id, bc_check_id=bc_check_id, severity=severity, report_type=CheckType.SECRETS):
                    continue
                result: _CheckResult = {'result': CheckResult.FAILED}
                try:
                    line_text = linecache.getline(secret.filename, secret.line_number)
                except SyntaxError as e:
                    # If encoding is a problem, this is probably not human-readable source code
                    # hence there's no need in flagging this secret
                    logging.info(f'Failed to log secret {secret.type} for file {secret.filename} because of {e}')
                    continue
                if line_text and line_text.startswith('git_commit'):
                    continue
                result = self.search_for_suppression(
                    check_id=check_id,
                    bc_check_id=bc_check_id,
                    severity=severity,
                    secret=secret,
                    runner_filter=runner_filter,
                ) or result
                report.add_resource(f'{secret.filename}:{secret.secret_hash}')
                # 'secret.secret_value' can actually be 'None', but only when 'PotentialSecret' was created
                # via 'load_secret_from_dict'
                line_text_censored = omit_secret_value_from_line(cast(str, secret.secret_value), line_text)
                report.add_record(Record(
                    check_id=check_id,
                    bc_check_id=bc_check_id,
                    severity=severity,
                    check_name=secret.type,
                    check_result=result,
                    code_block=[(secret.line_number, line_text_censored)],
                    file_path=f'/{os.path.relpath(secret.filename, root_folder)}',
                    file_line_range=[secret.line_number, secret.line_number + 1],
                    resource=secret.secret_hash,
                    check_class="",
                    evaluations=None,
                    file_abs_path=os.path.abspath(secret.filename),
                ))
            return report

    @staticmethod
    def _scan_files(files_to_scan: list[str], secrets: SecretsCollection, pbar: ProgressBar) -> None:
        # implemented the scan function like secrets.scan_files
        base_path = secrets.root
        results = parallel_runner.run_function(
            func=lambda f: Runner._safe_scan(f, base_path),
            items=files_to_scan,
            run_multiprocess=os.getenv("RUN_SECRETS_MULTIPROCESS", "").lower() == "true"
        )

        for filename, secrets_results in results:
            pbar.set_additional_data({'Current File Scanned': str(filename)})
            for secret in secrets_results:
                secrets[os.path.relpath(secret.filename, base_path)].add(secret)
            pbar.update()

    @staticmethod
    def _safe_scan(file_path: str, base_path: str) -> tuple[str, list[PotentialSecret]]:
        full_file_path = os.path.join(base_path, file_path)
        file_size = os.path.getsize(full_file_path)
        if file_size > MAX_FILE_SIZE > 0:
            logging.info(
                f'Skipping secret scanning on {full_file_path} due to file size. To scan this file for '
                'secrets, run this command again with the environment variable "CHECKOV_MAX_FILE_SIZE" '
                f'to 0 or {file_size + 1}'
            )
            return file_path, []
        try:
            start_time = datetime.datetime.now()
            file_results = [*scan.scan_file(full_file_path)]
            logging.info(f'file {full_file_path} results {file_results}')
            end_time = datetime.datetime.now()
            run_time = end_time - start_time
            if run_time > datetime.timedelta(seconds=10):
                logging.info(f'Secret scanning for {full_file_path} took {run_time} seconds')
            return file_path, file_results
        except Exception as e:
            logging.warning(f"Secret scanning: could not process file {full_file_path}")
            logging.debug(e, exc_info=True)
            return file_path, []

    @staticmethod
    def search_for_suppression(
            check_id: str,
            bc_check_id: str,
            severity: Severity | None,
            secret: PotentialSecret,
            runner_filter: RunnerFilter
    ) -> _CheckResult | None:
        if not runner_filter.should_run_check(check_id=check_id, bc_check_id=bc_check_id, severity=severity, report_type=CheckType.SECRETS) and check_id in CHECK_ID_TO_SECRET_TYPE.keys():
            return {
                "result": CheckResult.SKIPPED,
                "suppress_comment": f"Secret scan {check_id} is skipped"
            }
        # Check for suppression comment in the line before, the line of, and the line after the secret
        for line_number in [secret.line_number, secret.line_number - 1, secret.line_number + 1]:
            lt = linecache.getline(secret.filename, line_number)
            skip_search = re.search(COMMENT_REGEX, lt)
            if skip_search and (skip_search.group(2) == check_id or skip_search.group(2) == bc_check_id):
                return {
                    "result": CheckResult.SKIPPED,
                    "suppress_comment": skip_search.group(3)[1:] if skip_search.group(3) else "No comment provided"
                }
        return None



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction


class RootUser(BaseDockerfileCheck):
    def __init__(self) -> None:
        name = "Ensure the last USER is not root"
        id = "CKV_DOCKER_8"
        supported_instructions = ("USER",)
        categories = (CheckCategories.APPLICATION_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_resource_conf(self, conf: list[_Instruction]) -> tuple[CheckResult, list[_Instruction] | None]:
        last_user = conf[-1]
        if last_user["value"] == "root":
            return CheckResult.FAILED, [last_user]

        return CheckResult.PASSED, [last_user]


check = RootUser()



from __future__ import annotations

import json
import logging
import os
from abc import abstractmethod
from pathlib import Path
from typing import Any

import urllib3

from checkov.common.util.data_structures_utils import merge_dicts
from checkov.common.util.http_utils import get_user_agent_header


class BaseVCSDAL:
    def __init__(self) -> None:
        self.api_url = ""
        self.graphql_api_url = ""
        self.token = ""  # nosec
        self.current_repository = ""
        self.current_branch = ""
        self.repo_owner = ""
        self.org = ""
        self.default_branch_cache: dict[str, Any] = {}

        self.request_lib_http = None
        self._organization_security = None
        self.http: urllib3.PoolManager | None = None
        self.setup_http_manager(ca_certificate=os.getenv('BC_CA_BUNDLE', None))
        self.discover()
        self.setup_conf_dir()

    @abstractmethod
    def discover(self) -> None:
        """
            discover parameters from execution context of checkov. usually from env variable
        """
        self.default_branch_cache = {}

    def setup_http_manager(self, ca_certificate: str | None = None) -> None:
        """
        bridgecrew uses both the urllib3 and requests libraries, while checkov uses the requests library.
        :param ca_certificate: an optional CA bundle to be used by both libraries.
        """
        if self.http:
            return
        if ca_certificate:
            os.environ['REQUESTS_CA_BUNDLE'] = ca_certificate
            try:
                parsed_url = urllib3.util.parse_url(os.environ['https_proxy'])
                self.http = urllib3.ProxyManager(os.environ['https_proxy'], cert_reqs='REQUIRED',
                                                 ca_certs=ca_certificate,
                                                 proxy_headers=urllib3.make_headers(proxy_basic_auth=parsed_url.auth))  # type:ignore[no-untyped-call]
            except KeyError:
                self.http = urllib3.PoolManager(cert_reqs='REQUIRED', ca_certs=ca_certificate)
        else:
            try:
                parsed_url = urllib3.util.parse_url(os.environ['https_proxy'])
                self.http = urllib3.ProxyManager(os.environ['https_proxy'],
                                                 proxy_headers=urllib3.make_headers(proxy_basic_auth=parsed_url.auth))  # type:ignore[no-untyped-call]
            except KeyError:
                self.http = urllib3.PoolManager()

    def _request(self, endpoint: str, allowed_status_codes: list[int]) -> dict[str, Any] | None:
        if allowed_status_codes is None:
            allowed_status_codes = [200]
        if not self.token:
            return None
        url_endpoint = f"{self.api_url}/{endpoint}"
        try:
            headers = self._headers()
            if self.http:
                request = self.http.request("GET", url_endpoint, headers=headers)  # type:ignore[no-untyped-call]
                if request.status in allowed_status_codes:
                    data: dict[str, Any] = json.loads(request.data.decode("utf8"))
                    if isinstance(data, dict) and 'errors' in data.keys():
                        return None
                    return data
        except Exception:
            logging.debug(f"Query failed to run by returning code of {url_endpoint}", exc_info=True)
        return None

    @abstractmethod
    def _headers(self) -> dict[str, Any]:
        return merge_dicts({"Accept": "application/vnd.github.v3+json",
                            "Authorization": f"token {self.token}"}, get_user_agent_header())

    def _graphql_headers(self) -> dict[str, str]:
        return {"Authorization": f"bearer {self.token}"}

    def _request_graphql(self, query: str, variables: dict[str, Any]) -> Any:
        if not self.token:
            return
        headers = self._graphql_headers()

        body = json.dumps({'query': query, 'variables': variables})
        try:
            if self.http:
                request = self.http.request("POST", self.graphql_api_url, body=body, headers=headers)  # type:ignore[no-untyped-call]
                if request.status == 200:
                    data = json.loads(request.data.decode("utf8"))
                    if isinstance(data, dict) and 'errors' in data.keys():
                        logging.debug("received errors %s", data)
                        return None
                    return data
                else:
                    logging.debug("Query failed to run by returning code of {}. {}".format(request.data, query))
        except Exception:
            logging.debug(f"Query failed {query}", exc_info=True)

    @staticmethod
    def persist(path: str | Path, conf: dict[str, Any] | list[dict[str, Any]]) -> None:
        BaseVCSDAL.ensure_dir(path)
        with open(path, "w+", encoding='utf-8') as f:
            logging.debug(f"Persisting to {path}")
            json.dump(conf, f, ensure_ascii=False, indent=4)

    @staticmethod
    def ensure_dir(file_path: str | Path) -> None:
        if not os.path.exists(file_path):
            directory_path = os.path.dirname(file_path)
            if not os.path.exists(directory_path):
                os.makedirs(directory_path)

    @abstractmethod
    def setup_conf_dir(self) -> None:
        pass



from __future__ import annotations
from typing import Any

from checkov.common.images.image_referencer import Image
from checkov.github_actions.image_referencer.provider import GithubActionProvider


class GithubActionsImageReferencerManager:
    __slots__ = ("workflow_config", "file_path", "workflow_line_numbers")

    def __init__(self, workflow_config: dict[str, Any], file_path: str, workflow_line_numbers: list[tuple[int, str]]):
        self.workflow_config = workflow_config
        self.file_path = file_path
        self.workflow_line_numbers = workflow_line_numbers

    def extract_images_from_workflow(self) -> list[Image]:
        provider = GithubActionProvider(workflow_config=self.workflow_config, file_path=self.file_path,
                                        workflow_line_numbers=self.workflow_line_numbers)
        images: list[Image] = provider.extract_images_from_workflow()

        return images



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class ECSTaskDefinitionEFSVolumeEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Encryption in transit is enabled for EFS volumes in ECS Task definitions"
        id = "CKV_AWS_97"
        supported_resources = ("AWS::ECS::TaskDefinition",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("Properties")
        if properties and isinstance(properties, dict):
            volumes = properties.get("Volumes")
            if volumes and isinstance(volumes, list):
                for volume in volumes:
                    efs_config = volume.get("EFSVolumeConfiguration")
                    if efs_config and isinstance(efs_config, dict):
                        if efs_config.get("TransitEncryption") == "ENABLED":
                            return CheckResult.PASSED
                        else:
                            return CheckResult.FAILED

        return CheckResult.PASSED


check = ECSTaskDefinitionEFSVolumeEncryption()



from __future__ import annotations

from typing import Any

from checkov.azure_pipelines.checks.base_azure_pipelines_check import BaseAzurePipelinesCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.yaml_doc.enums import BlockType


class ContainerLatestTag(BaseAzurePipelinesCheck):
    def __init__(self) -> None:
        name = "Ensure container job uses a non latest version tag"
        id = "CKV_AZUREPIPELINES_1"
        super().__init__(
            name=name,
            id=id,
            categories=(CheckCategories.SUPPLY_CHAIN,),
            supported_entities=("jobs", "stages[].jobs[]"),
            block_type=BlockType.ARRAY,
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        container = conf.get("container")
        if container and isinstance(container, str):
            if ":" in container:
                # some image tag
                if container.split(":")[1] == "latest":
                    # latest image tag
                    return CheckResult.FAILED, conf
            elif "@" not in container:
                # no image tag
                return CheckResult.FAILED, conf

            # image tag is either not latest or a digest
            return CheckResult.PASSED, conf

        return CheckResult.UNKNOWN, conf


check = ContainerLatestTag()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import List, Any


class SpaceBucketPublicRead(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure the Spaces bucket is private"
        id = "CKV_DIO_3"
        supported_resources = ['digitalocean_spaces_bucket']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories,
                         supported_resources=supported_resources,
                         missing_attribute_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return "acl"

    def get_forbidden_values(self) -> List[Any]:
        return ["public-read"]


check = SpaceBucketPublicRead()



from __future__ import annotations

import logging
import subprocess  # nosec
from pathlib import Path
from typing import Union, Dict, Any
import asyncio
from urllib.parse import quote_plus

import docker
import json
import os
import time

from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import \
    docker_image_scanning_integration
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.util.file_utils import decompress_file_gzip_base64
from checkov.common.util.http_utils import request_wrapper
from checkov.common.bridgecrew.platform_key import bridgecrew_dir

TWISTCLI_FILE_NAME = 'twistcli'
DOCKER_IMAGE_SCAN_RESULT_FILE_NAME = 'docker-image-scan-results.json'
CHECKOV_SEC_IN_WEEK = 604800


def generate_image_name() -> str:
    return f'repository/image{str(time.time() * 1000)}'


def _get_docker_image_name(docker_image_id: str) -> str:
    try:
        docker_client = docker.from_env()
        image_name: str = docker_client.images.get(docker_image_id).attrs['RepoDigests'][0].split('@')[0]
        return image_name
    except Exception:
        logging.info("Failed to fetch image name.", exc_info=True)
        return generate_image_name()


def _get_dockerfile_content(dockerfile_path: Union[str, "os.PathLike[str]"]) -> str:
    try:
        with open(dockerfile_path) as f:
            return f.read()
    except FileNotFoundError:
        logging.error("Path to Dockerfile is invalid", exc_info=True)
        raise
    except Exception:
        logging.error("Failed to read Dockerfile content", exc_info=True)
        raise


class ImageScanner:
    def __init__(self) -> None:
        self.docker_image_name = ''
        self.dockerfile_content = ''
        self.twistcli_path = Path(bridgecrew_dir) / TWISTCLI_FILE_NAME

    def setup_scan(
            self,
            docker_image_id: str,
            dockerfile_path: Union[str, "os.PathLike[str]"],
            skip_extract_image_name: bool,
    ) -> None:
        if skip_extract_image_name:
            # Provide a default image name in case the image has not been tagged with a name
            self.docker_image_name = generate_image_name()
        else:
            self.docker_image_name = _get_docker_image_name(docker_image_id)
        self.dockerfile_content = _get_dockerfile_content(dockerfile_path)

        self.setup_twistcli()

    def cleanup_scan(self) -> None:
        if self.twistcli_path.exists():
            os.remove(self.twistcli_path)
            logging.info('twistcli file removed')

    def run_image_scan(self, docker_image_id: str) -> Dict[str, Any]:
        command = f"./{self.twistcli_path} images scan --address {docker_image_scanning_integration.get_proxy_address()} --token {docker_image_scanning_integration.get_bc_api_key()} --details --output-file \"{DOCKER_IMAGE_SCAN_RESULT_FILE_NAME}\" {docker_image_id}"
        logging.debug(f"TwistCLI: {command}")
        try:
            subprocess.run([command], check=True, shell=True)  # nosec
        except Exception as exc:
            logging.error("Failed to scan image", exc_info=True)
            self.cleanup_scan()
            raise Exception(f"Failed to scan image {docker_image_id}") from exc
        self.cleanup_scan()
        logging.info(f'TwistCLI ran successfully on image {docker_image_id}')

        with open(DOCKER_IMAGE_SCAN_RESULT_FILE_NAME) as docker_image_scan_result_file:
            scan_result: dict[str, Any] = json.load(docker_image_scan_result_file)
        return scan_result

    def scan(self, docker_image_id: str, dockerfile_path: str, skip_extract_image_name: bool = False) -> int:
        try:
            self.setup_scan(docker_image_id, dockerfile_path, skip_extract_image_name)
            scan_result = self.run_image_scan(docker_image_id)
            exit_code = asyncio.run(
                docker_image_scanning_integration.report_results_async(
                    twistcli_scan_result=scan_result,
                    bc_platform_integration=bc_integration,
                    bc_api_key=docker_image_scanning_integration.get_bc_api_key(),
                    file_path=Path(dockerfile_path),
                    file_content=self.dockerfile_content,
                    docker_image_name=self.docker_image_name,
                )
            )

            logging.info('Docker image scanning results reported to the platform')
            return exit_code
        except Exception:
            logging.error("Failed to scan docker image", exc_info=True)
            return 1

    @staticmethod
    def get_scan_results_from_cache(image_id: str) -> Dict[str, Any]:
        image_id_encode = quote_plus(image_id)

        try:
            response = request_wrapper(
                "GET", f"{bc_integration.api_url}/api/v1/vulnerabilities/scan-results/{image_id_encode}",
                headers=bc_integration.get_default_headers("GET"), should_call_raise_for_status=True
            )
            response_json = response.json()
            output_type = response_json["outputType"]

            logging.info(f"{output_type} returned from cache")

            if output_type == "Error":
                logging.error(response_json["outputData"])
                return {}

            elif output_type == "Result":
                result: dict[str, Any] = json.loads(
                    decompress_file_gzip_base64(
                        response_json["outputData"]
                    )
                )
                return result

            logging.info(f"Got an empty result for image={image_id}")
        except Exception:
            logging.info("Failed to retrieve image cache result", exc_info=True)

        return {}

    def should_download(self) -> bool:
        if not self.twistcli_path.exists():
            return True
        last_modification = os.stat(self.twistcli_path)
        file_age = (time.time() - last_modification.st_mtime)
        return file_age >= int(os.getenv("CHECKOV_EXPIRATION_TIME_IN_SEC", CHECKOV_SEC_IN_WEEK))

    def setup_twistcli(self) -> None:
        try:
            if self.should_download():
                if not os.path.exists(bridgecrew_dir):
                    try:
                        os.makedirs(bridgecrew_dir)
                    except FileExistsError:
                        # In multi-processing, this might meet a race condition
                        pass
                self.cleanup_scan()
                docker_image_scanning_integration.download_twistcli(image_scanner.twistcli_path)
        except Exception:
            logging.error("Failed to setup docker image scanning", exc_info=True)
            raise


image_scanner = ImageScanner()



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.terraform.image_referencer.provider.aws import AwsTerraformProvider
from checkov.terraform.image_referencer.provider.azure import AzureTerraformProvider

if TYPE_CHECKING:
    from checkov.common.images.image_referencer import Image
    from networkx import DiGraph


class TerraformImageReferencerManager:
    __slots__ = ("graph_connector",)

    def __init__(self, graph_connector: DiGraph) -> None:
        self.graph_connector = graph_connector

    def extract_images_from_resources(self) -> list[Image]:
        images = []

        aws_provider = AwsTerraformProvider(graph_connector=self.graph_connector)
        azure_provider = AzureTerraformProvider(graph_connector=self.graph_connector)

        images.extend(aws_provider.extract_images_from_resources())
        images.extend(azure_provider.extract_images_from_resources())

        return images



from __future__ import annotations

import logging
from copy import deepcopy
from typing import TYPE_CHECKING, Any

from checkov.bicep.graph_builder.graph_components.block_types import BlockType
from checkov.common.graph.graph_builder import Edge, CustomAttributes
from checkov.common.graph.graph_builder.graph_components.blocks import Block
from checkov.common.graph.graph_builder.local_graph import LocalGraph
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.dockerfile.graph_builder.graph_components.resource_types import ResourceType
from checkov.dockerfile.utils import DOCKERFILE_STARTLINE, DOCKERFILE_ENDLINE

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction  # only in extra_stubs
    from checkov.common.graph.graph_builder.local_graph import _Block


class DockerfileLocalGraph(LocalGraph[Block]):
    def __init__(self, definitions: dict[str, dict[str, list[_Instruction]]]) -> None:
        super().__init__()
        self.vertices: list[Block] = []
        self.definitions = definitions
        self.vertices_by_path_and_name: dict[tuple[str, str], int] = {}

    def build_graph(self, render_variables: bool = False) -> None:
        self._create_vertices()
        logging.debug(f"[DockerfileLocalGraph] created {len(self.vertices)} vertices")

        for i, vertex in enumerate(self.vertices):
            self.vertices_by_block_type[vertex.block_type].append(i)
            self.vertices_block_name_map[vertex.block_type][vertex.name].append(i)
            self.vertices_by_path_and_name[(vertex.path, vertex.name)] = i

            self.in_edges[i] = []
            self.out_edges[i] = []

        self._create_edges()
        logging.debug(f"[DockerfileLocalGraph] created {len(self.edges)} edges")

    def _create_vertices(self) -> None:
        for file_path, definition in self.definitions.items():
            for instruction_type, instructions in definition.items():
                self._create_instruction_vertices(
                    file_path=file_path,
                    instruction_type=instruction_type,
                    instructions=instructions,
                )

    def _create_instruction_vertices(
        self, file_path: str, instruction_type: str, instructions: list[_Instruction]
    ) -> None:
        """Creates supported 'instruction_type' vertices"""

        if instruction_type == "COMMENT":
            # not interested in comments
            return

        for instruction in instructions:
            resource_type = ResourceType.__dict__.get(instruction_type)
            if not resource_type:
                logging.warning(f"An unsupported instruction {instruction_type} was used in {file_path}")
                continue

            config = {
                "content": instruction["content"],
                "value": instruction["value"],
                START_LINE: instruction[DOCKERFILE_STARTLINE],
                END_LINE: instruction[DOCKERFILE_ENDLINE],
            }

            attributes = deepcopy(config)
            attributes[CustomAttributes.RESOURCE_TYPE] = resource_type

            self.vertices.append(
                Block(
                    name=resource_type,
                    config=config,
                    path=file_path,
                    block_type=BlockType.RESOURCE,
                    attributes=attributes,
                    id=resource_type,
                    source="Dockerfile",
                )
            )

    def _create_edges(self) -> None:
        pass

    def _create_edge(self, origin_vertex_index: int, dest_vertex_index: int, label: str = "default") -> None:
        if origin_vertex_index == dest_vertex_index:
            # this should not happen
            return

        edge = Edge(origin_vertex_index, dest_vertex_index, label)
        self.edges.append(edge)
        self.out_edges[origin_vertex_index].append(edge)
        self.in_edges[dest_vertex_index].append(edge)

    def update_vertices_configs(self) -> None:
        # not used
        pass

    @staticmethod
    def update_vertex_config(
        vertex: _Block, changed_attributes: list[str] | dict[str, Any], has_dynamic_blocks: bool = False
    ) -> None:
        # not used
        pass

    def get_resources_types_in_graph(self) -> list[str]:
        # not used
        return []



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class DynamoDBTableReplicaKMSUsesCMK(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure DynamoDB table replica KMS encryption uses CMK"
        id = "CKV_AWS_271"
        supported_resources = ('aws_dynamodb_table_replica',)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'kms_key_arn'

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = DynamoDBTableReplicaKMSUsesCMK()



from __future__ import annotations

import os
import shutil
from typing import Any
from pathlib import Path

from checkov.common.runners.base_runner import strtobool
from checkov.common.vcs.base_vcs_dal import BaseVCSDAL
from checkov.github.schemas.org_security import schema as org_security_schema


class Github(BaseVCSDAL):
    github_conf_dir_path: str  # noqa: CCE003  # a static attribute
    github_conf_file_paths: dict[str, list[Path]]  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__()

    def setup_conf_dir(self) -> None:
        github_conf_dir_name = os.getenv('CKV_GITHUB_CONF_DIR_NAME', 'github_conf')
        self.github_conf_dir_path = os.path.join(os.getcwd(), github_conf_dir_name)
        os.environ["CKV_GITHUB_CONF_DIR_PATH"] = self.github_conf_dir_path

        # if any file was left from previous run-remove it.
        if os.path.isdir(self.github_conf_dir_path):
            shutil.rmtree(self.github_conf_dir_path)

        self.github_conf_file_paths = {
            "org_security": [Path(self.github_conf_dir_path) / "org_security.json"],
            "branch_protection_rules": [Path(self.github_conf_dir_path) / "branch_protection_rules.json"],
            "org_webhooks": [],  # is updated when persisted
            "repository_webhooks": [],  # is updated when persisted
            "repository_collaborators": [Path(self.github_conf_dir_path) / "repository_collaborators.json"],
            "branch_metadata": [Path(self.github_conf_dir_path) / "branch_metadata.json"],
            "org_metadata": [Path(self.github_conf_dir_path) / "org_metadata.json"],
        }

    def discover(self) -> None:
        self.api_url = os.getenv('GITHUB_API_URL', "https://api.github.com")
        self.graphql_api_url = f"{self.api_url}/graphql"

        self.token = os.getenv('GITHUB_TOKEN', '')
        self.repo_owner = os.getenv('GITHUB_REPO_OWNER', '')
        self.current_repository = os.getenv('GITHUB_REPOSITORY', '')
        self.current_branch = os.getenv('GITHUB_REF_NAME', '')
        if not self.current_branch:
            self.current_branch = os.getenv('GITHUB_REF', 'refs/heads/master')
            if self.current_branch:
                extracted_branch_array = self.current_branch.split("/")
                if len(extracted_branch_array) == 3:
                    self.current_branch = extracted_branch_array[2]

        self.default_branch_cache = {}
        self.org = os.getenv('GITHUB_ORG', '')

    def _headers(self) -> dict[str, str]:
        return {"Accept": "application/vnd.github.v3+json",
                "Authorization": "token {}".format(self.token)}

    # -------------------------------- Endpoints -------------------------------- #

    def get_branch_protection_rules(self) -> dict[str, Any] | None:
        if self.current_branch and self.current_repository:
            data = self._request(
                endpoint=f"repos/{self.repo_owner}/{self.current_repository}/branches/{self.current_branch}/protection",
                allowed_status_codes=[200, 404])
            return data
        return None

    def get_organization_webhooks(self) -> list[dict[str, Any]] | None:
        data = self._request(endpoint=f"orgs/{self.org}/hooks", allowed_status_codes=[200])
        if isinstance(data, list):
            return data
        return None

    def get_repository_collaborators(self) -> dict[str, Any] | None:
        data = self._request(
            endpoint=f"repos/{self.repo_owner}/{self.current_repository}/collaborators",
            allowed_status_codes=[200]
        )
        return data

    def get_repository_webhooks(self) -> list[dict[str, Any]] | None:
        data = self._request(
            endpoint=f"repos/{self.repo_owner}/{self.current_repository}/hooks",
            allowed_status_codes=[200])
        if isinstance(data, list):
            return data
        return None

    def get_organization_security(self) -> dict[str, str] | None:
        if not self._organization_security:
            data = self._request_graphql(query="""
                query ($org: String! ) {
                    organization(login: $org) {
                        name
                        login
                        description
                        ipAllowListEnabledSetting
                        ipAllowListForInstalledAppsEnabledSetting
                        requiresTwoFactorAuthentication
                        samlIdentityProvider {
                            ssoUrl
                        }
                    }
                }
                """, variables={'org': self.org})
            if not data:
                return None
            if org_security_schema.validate(data):
                self._organization_security = data
        return self._organization_security

    def get_branch_metadata(self) -> dict[str, Any] | None:
        # new endpoint since Dec22
        data = self._request(
            endpoint=f"repos/{self.repo_owner}/{self.current_repository}/branches/{self.current_branch}",
            allowed_status_codes=[200]
        )
        return data

    def get_organization_metadata(self) -> dict[str, Any] | None:
        # new endpoint since Dec22
        data = self._request(endpoint=f"orgs/{self.org}", allowed_status_codes=[200])
        return data

    # --------------------------------------------------------------------------- #

    def persist_branch_protection_rules(self) -> None:
        data = self.get_branch_protection_rules()
        if data:
            BaseVCSDAL.persist(path=self.github_conf_file_paths["branch_protection_rules"][0], conf=data)

    def persist_organization_security(self) -> None:
        organization_security = self.get_organization_security()
        if organization_security:
            BaseVCSDAL.persist(path=self.github_conf_file_paths["org_security"][0], conf=organization_security)

    def persist_organization_webhooks(self) -> None:
        organization_webhooks = self.get_organization_webhooks()
        if organization_webhooks:
            for idx, item in enumerate(organization_webhooks):
                path = Path(self.github_conf_dir_path) / f"org_webhooks{idx+1}.json"
                self.github_conf_file_paths["org_webhooks"].append(path)
                BaseVCSDAL.persist(path=path, conf=[item])

    def persist_repository_collaborators(self) -> None:
        repository_collaborators = self.get_repository_collaborators()
        if repository_collaborators:
            BaseVCSDAL.persist(
                path=self.github_conf_file_paths["repository_collaborators"][0],
                conf=repository_collaborators)

    def persist_repository_webhooks(self) -> None:
        repository_webhooks = self.get_repository_webhooks()
        if repository_webhooks:
            for idx, item in enumerate(repository_webhooks):
                path = Path(self.github_conf_dir_path) / f"repository_webhooks{idx + 1}.json"
                self.github_conf_file_paths["repository_webhooks"].append(path)
                BaseVCSDAL.persist(path=path, conf=[item])

    def persist_branch_metadata(self) -> None:
        branch_metadata = self.get_branch_metadata()
        if branch_metadata:
            BaseVCSDAL.persist(path=self.github_conf_file_paths["branch_metadata"][0], conf=branch_metadata)

    def persist_organization_metadata(self) -> None:
        org_metadata = self.get_organization_metadata()
        if org_metadata:
            BaseVCSDAL.persist(path=self.github_conf_file_paths["org_metadata"][0], conf=org_metadata)

    def persist_all_confs(self) -> None:
        if strtobool(os.getenv("CKV_GITHUB_CONFIG_FETCH_DATA", "True")):
            self.persist_organization_security()
            self.persist_branch_protection_rules()
            self.persist_organization_webhooks()
            self.persist_repository_webhooks()
            self.persist_repository_collaborators()
            self.persist_branch_metadata()
            self.persist_organization_metadata()



from typing import Optional, Any, Dict
from checkov.common.checks_infra.solvers.attribute_solvers.range_includes_attribute_solver import RangeIncludesAttributeSolver
from checkov.common.graph.checks_infra.enums import Operators


class RangeNotIncludesAttributeSolver(RangeIncludesAttributeSolver):
    operator = Operators.RANGE_NOT_INCLUDES  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        return not super()._get_operation(vertex, attribute)



from __future__ import annotations

import os
from typing import TYPE_CHECKING, Callable, Any

from hcl2 import START_LINE, END_LINE

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.images.image_referencer import Image
from checkov.common.util.str_utils import removeprefix
from checkov.terraform.graph_builder.utils import setup_file_path_to_referred_id, get_related_resource_id

if TYPE_CHECKING:
    from networkx import DiGraph
    from typing_extensions import TypeAlias

_ExtractImagesCallableAlias: TypeAlias = Callable[["dict[str, Any]"], "list[str]"]


class BaseTerraformProvider:
    __slots__ = ("graph_connector", "supported_resource_types")

    def __init__(
        self, graph_connector: DiGraph, supported_resource_types: dict[str, _ExtractImagesCallableAlias]
    ) -> None:
        self.graph_connector = graph_connector
        self.supported_resource_types = supported_resource_types

    def extract_images_from_resources(self) -> list[Image]:
        images = []

        resource_nodes = [
            node
            for node, resource_type in self.graph_connector.nodes(data=CustomAttributes.RESOURCE_TYPE)
            if resource_type and resource_type in self.supported_resource_types
        ]

        supported_resources_graph = self.graph_connector.subgraph(resource_nodes)
        file_path_to_referred_id = setup_file_path_to_referred_id(self.graph_connector)

        for _, resource in supported_resources_graph.nodes(data=True):
            image_names: list[str] = []
            resource_type = resource[CustomAttributes.RESOURCE_TYPE]

            extract_images_func = self.supported_resource_types.get(resource_type)
            if extract_images_func:
                image_names.extend(extract_images_func(resource))

            related_resource_id = get_related_resource_id(resource, file_path_to_referred_id)
            for name in image_names:
                images.append(
                    Image(
                        file_path=resource[CustomAttributes.FILE_PATH],
                        name=name,
                        start_line=resource[START_LINE],
                        end_line=resource[END_LINE],
                        related_resource_id=f'{removeprefix(resource.get("file_path_"), os.getenv("BC_ROOT_DIR", ""))}:{related_resource_id}'
                    )
                )

        return images



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.parsers.yaml.parser import parse
from checkov.common.runners.object_runner import Runner as ObjectRunner
from checkov.common.util.consts import START_LINE, END_LINE

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
    from checkov.common.runners.graph_builder.local_graph import ObjectLocalGraph
    from checkov.common.runners.graph_manager import ObjectGraphManager


class Runner(ObjectRunner):
    check_type = CheckType.YAML  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        db_connector: NetworkxConnector | None = None,
        source: str = "yaml",
        graph_class: type[ObjectLocalGraph] | None = None,
        graph_manager: ObjectGraphManager | None = None,
    ) -> None:
        super().__init__(
            db_connector=db_connector,
            source=source,
            graph_class=graph_class,
            graph_manager=graph_manager,
        )
        self.file_extensions = ['.yaml', '.yml']

    def import_registry(self) -> BaseCheckRegistry:
        from checkov.yaml_doc.registry import registry

        return registry

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        return parse(f, file_content)

    def get_start_end_lines(
        self, end: int, result_config: dict[str, Any] | list[dict[str, Any]], start: int
    ) -> tuple[int, int]:
        if result_config and isinstance(result_config, list):
            if not isinstance(result_config[0], dict):
                return -1, -1
            start = result_config[0]["__startline__"] - 1
            end = result_config[len(result_config) - 1]["__endline__"]
        elif result_config and isinstance(result_config, dict):
            start = result_config["__startline__"]
            end = result_config["__endline__"]
        return end, start

    @staticmethod
    def resolve_sub_name(definition: dict[str, Any], start_line: int, end_line: int, tag: str) -> str:
        """
        extract the value of the tag, that is within the line of range(start_line, end_line)

        >>> Runner.resolve_sub_name({"executors":{"image-executor":{"docker":[],"__startline__":8,"__endline__":11}}}, 9, 11, 'executors')
        'image-executor'

        >>> Runner.resolve_sub_name({"jobs":{"job-name":{"docker":[],"__startline__":13,"__endline__":20}}}, 15, 16, 'jobs')
        'job-name'
        """
        if not definition:
            return ""
        for key, sub_name in definition.get(tag, {}).items():
            if key in (START_LINE, END_LINE):
                continue
            if sub_name[START_LINE] <= start_line <= end_line <= sub_name[END_LINE]:
                return str(key)
        return ""

    @staticmethod
    def resolve_step_name(job_definition: dict[str, Any], start_line: int, end_line: int) -> str:
        """
        extract the step name from the given job within the line of range(start_line, end_line)

        >>> Runner.resolve_step_name({"steps":["checkout",{}],"__startline__":42,"__endline__":49}, 48, 49)
        '[1](checkout)'

        >>> Runner.resolve_step_name({"runs-on":"ubuntu-latest","steps":[{"uses":"actions/checkout@v2","__startline__":22,"__endline__":23}]}, 22, 23)
        '[1]'

        >>> Runner.resolve_step_name({"runs-on":"ubuntu-latest","steps":[{"name": "ab","__startline__":22,"__endline__":23}, {"name":"step_name","__startline__":23,"__endline__":33}]}, 23, 33)
        '[2](step_name)'

        """
        if not job_definition:
            return ""
        for idx, step in enumerate([step for step in job_definition.get('steps') or [] if step]):
            if isinstance(step, str):
                return f"[{idx + 1}]({step})"
            elif isinstance(step, dict):
                if step[START_LINE] <= start_line <= end_line <= step[END_LINE]:
                    name = step.get('name')
                    return f"[{idx + 1}]({name})" if name else f"[{idx + 1}]"
        return ""

    @staticmethod
    def resolve_image_name(image_definition: dict[str, Any], start_line: int, end_line: int) -> str:
        """
        extract the image name from the given job definition within the line of range(start_line, end_line)

        >>> Runner.resolve_image_name({"docker":[{"image":"mongo:2.6.8","__startline__":15,"__endline__":16}]}, 15, 16)
        '[1](mongo:2.6.8)'

        """
        if not image_definition:
            return ""
        for idx, step in enumerate([step for step in image_definition.get('docker') or [] if step]):
            if isinstance(image_definition.get('docker'), dict):
                if step == 'image':
                    return f"[{idx + 1}]({image_definition['docker'][step]})"
            if isinstance(step, str):
                return f"[{idx + 1}]({step})"
            elif isinstance(step, dict):
                if step[START_LINE] <= start_line <= end_line <= step[END_LINE]:
                    name = step.get('image')
                    return f"[{idx + 1}]({name})" if name else f"[{idx + 1}]"
        return ""



from checkov.terraform.checks.resource.aws.AbsNACLUnrestrictedIngress import AbsNACLUnrestrictedIngress


class NACLUnrestrictedIngress22(AbsNACLUnrestrictedIngress):
    def __init__(self):
        super().__init__(check_id="CKV_AWS_232", port=22)


check = NACLUnrestrictedIngress22()



from __future__ import annotations

import re
from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.repository_webhooks import schema as repository_webhooks_schema
from checkov.json_doc.enums import BlockType


class WebhookHttpsRepo(BaseGithubCheck):
    def __init__(self) -> None:
        name = "Ensure GitHub repository webhooks are using HTTPS"
        id = "CKV_GITHUB_7"
        categories = [CheckCategories.SUPPLY_CHAIN]
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=["*"],
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]] | None:  # type:ignore[override]
        if repository_webhooks_schema.validate(conf):
            for item in conf:
                if isinstance(item, dict):
                    item_config = item.get("config", {})
                    if not item_config:
                        continue
                    url = item_config.get('url', '')
                    insecure_ssl = item_config.get('insecure_ssl', '0')
                    if re.match("^http://", url) or insecure_ssl != '0':
                        return CheckResult.FAILED, item_config
            return CheckResult.PASSED, conf
        return CheckResult.UNKNOWN, conf


check = WebhookHttpsRepo()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.gitlab_ci.checks.base_gitlab_ci_check import BaseGitlabCICheck
from checkov.yaml_doc.enums import BlockType

PIPELINE_SOURCES = ('$CI_PIPELINE_SOURCE == "merge_request_event"', '$CI_PIPELINE_SOURCE == "push"')


class AvoidDoublePipelines(BaseGitlabCICheck):
    def __init__(self) -> None:
        name = "Avoid creating rules that generate double pipelines"
        id = "CKV_GITLABCI_2"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=('*.rules',)
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        c = 0

        for rule in conf:
            if isinstance(rule, dict) and "if" in rule:
                value = rule['if']
                if value.startswith(PIPELINE_SOURCES):
                    c += 1
                    if c > 1:
                        return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf
        

check = AvoidDoublePipelines()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.gitlab.base_gitlab_configuration_check import BaseGitlabCheck
from checkov.gitlab.schemas.project_approvals import schema as project_aprovals_schema
from checkov.json_doc.enums import BlockType


class MergeRequestRequiresApproval(BaseGitlabCheck):
    def __init__(self) -> None:
        name = "Merge requests should require at least 2 approvals"
        id = "CKV_GITLAB_1"
        categories = [CheckCategories.SUPPLY_CHAIN]
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_entities=["*"],
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> tuple[CheckResult, dict[str, Any]] | None:  # type:ignore[override]
        if project_aprovals_schema.validate(conf):
            if conf.get("approvals_before_merge", 0) < 2:
                return CheckResult.FAILED, conf
            return CheckResult.PASSED, conf
        return None


check = MergeRequestRequiresApproval()



from typing import Dict, Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.common.models.consts import ANY_VALUE
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class SQSQueueEncryption(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure all data stored in the SQS queue is encrypted"
        id = "CKV_AWS_27"
        supported_resources = ['aws_sqs_queue']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        if conf.get('sqs_managed_sse_enabled'):
            return CheckResult.PASSED
        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return 'kms_master_key_id'

    def get_expected_value(self) -> str:
        return ANY_VALUE


check = SQSQueueEncryption()



from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class ServerPublicIP(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure Server instance should not have public IP."
        id = "CKV_NCP_23"
        supported_resource = ('ncloud_public_ip',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resource)

    def get_inspected_key(self):
        return 'server_instance_no'

    def get_forbidden_values(self):
        return [ANY_VALUE]


check = ServerPublicIP()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class GoogleCloudSqlBackupConfiguration(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure all Cloud SQL database instance have backup configuration enabled"
        id = "CKV_GCP_14"
        supported_resources = ("google_sql_database_instance",)
        categories = (CheckCategories.BACKUP_AND_RECOVERY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if "master_instance_name" in conf:
            return CheckResult.UNKNOWN
        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return "settings/[0]/backup_configuration/[0]/enabled"


check = GoogleCloudSqlBackupConfiguration()



from __future__ import annotations

from typing import Any
from bc_jsonpath_ng import parse

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.github.base_github_configuration_check import BaseGithubCheck
from checkov.github.schemas.organization import schema as org_schema
from checkov.json_doc.enums import BlockType


class RepositoryCreationIsLimited(BaseGithubCheck):
    def __init__(self, id: str, name: str) -> None:
        # see https://docs.github.com/en/rest/orgs/orgs?apiVersion=2022-11-28#update-an-organization
        # https://developer.github.com/changes/2019-12-03-internal-visibility-changes/
        # members_allowed_repository_creation_type is deprecated.
        categories = (CheckCategories.SUPPLY_CHAIN,)
        super().__init__(
            id=id,
            name=name,
            categories=categories,
            supported_entities=("*",),
            block_type=BlockType.DOCUMENT
        )

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult | None:  # type:ignore[override]
        if org_schema.validate(conf):
            evaluated_key = self.get_evaluated_keys()[0].replace("/", ".")
            jsonpath_expression = parse(f"$..{evaluated_key}")
            matches = jsonpath_expression.find(conf)
            if matches and not matches[0].value:
                return CheckResult.PASSED
            # default value for all 3 public, internal and private is true.
            return CheckResult.FAILED
        return None



from __future__ import annotations

import itertools
from typing import Any, List, Dict, Optional, Tuple, TYPE_CHECKING

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.terraform.graph_builder.graph_components.block_types import BlockType

if TYPE_CHECKING:
    from networkx import DiGraph


class BaseConnectionSolver(BaseSolver):
    # resource is automatically supported
    SUPPORTED_CONNECTION_BLOCK_TYPES = (BlockType.OUTPUT,)

    def __init__(
        self,
        resource_types: List[str],
        connected_resources_types: List[str],
        vertices_under_resource_types: Optional[List[Dict[str, Any]]] = None,
        vertices_under_connected_resources_types: Optional[List[Dict[str, Any]]] = None,
    ) -> None:
        super().__init__(SolverType.CONNECTION)
        self.resource_types = resource_types
        self.connected_resources_types = connected_resources_types
        self.targeted_resources_types = set(itertools.chain(resource_types, connected_resources_types))
        self.vertices_under_resource_types = vertices_under_resource_types or []
        self.vertices_under_connected_resources_types = vertices_under_connected_resources_types or []
        self.excluded_vertices: List[Dict[str, Any]] = []
        self.unknown_vertices: List[Dict[str, Any]] = []

    def run(self, graph_connector: DiGraph) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        self.set_vertices(graph_connector, [], [])

        subgraph = self.reduce_graph_by_target_types(graph_connector)

        return self.get_operation(subgraph)

    def is_associated_edge(self, origin_type: str, destination_type: str) -> bool:
        return (origin_type in self.resource_types and destination_type in self.connected_resources_types) or (
            origin_type in self.connected_resources_types and destination_type in self.resource_types
        )

    def is_associated_vertex(self, vertex_type: str) -> bool:
        return vertex_type in itertools.chain(self.resource_types, self.connected_resources_types)

    def set_vertices(self, graph_connector: DiGraph, exclude_vertices: List[Dict[str, Any]], unknown_vertices: List[Dict[str, Any]]) -> None:
        self.vertices_under_resource_types = [
            v for _, v in graph_connector.nodes(data=True) if self.resource_type_pred(v, self.resource_types)
        ]
        self.vertices_under_connected_resources_types = [
            v for _, v in graph_connector.nodes(data=True) if self.resource_type_pred(v, self.connected_resources_types)
        ]
        self.excluded_vertices = [
            v
            for v in itertools.chain(self.vertices_under_resource_types, self.vertices_under_connected_resources_types)
            if v in exclude_vertices
        ]
        self.unknown_vertices = [
            v
            for v in itertools.chain(self.vertices_under_resource_types, self.vertices_under_connected_resources_types)
            if v in unknown_vertices
        ]

    def reduce_graph_by_target_types(self, graph_connector: DiGraph) -> DiGraph:
        # no need to create a subgraph, if there are no vertices to be checked
        if not self.vertices_under_resource_types:
            return graph_connector

        resource_nodes = {
            node
            for node, resource_type in graph_connector.nodes(data=CustomAttributes.RESOURCE_TYPE)
            if resource_type in self.targeted_resources_types
        }

        # tuple needs to be adjusted, if more connection block types are supported
        connection_nodes = {
            node
            for node, block_type in graph_connector.nodes(data=CustomAttributes.BLOCK_TYPE)
            if block_type in BaseConnectionSolver.SUPPORTED_CONNECTION_BLOCK_TYPES
        }
        resource_nodes.update(connection_nodes)

        return graph_connector.subgraph(resource_nodes)

    def get_operation(self, graph_connector: DiGraph) -> \
            Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        raise NotImplementedError

    def _get_operation(self, *args: Any, **kwargs: Any) -> Any:
        raise NotImplementedError



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction


class AddExists(BaseDockerfileCheck):
    def __init__(self) -> None:
        name = "Ensure that COPY is used instead of ADD in Dockerfiles"
        id = "CKV_DOCKER_4"
        supported_instructions = ("ADD",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_resource_conf(self, conf: list[_Instruction]) -> tuple[CheckResult, list[_Instruction] | None]:
        for instruction in conf:
            if instruction['instruction'] == "ADD":
                return CheckResult.FAILED, conf
        return CheckResult.PASSED, None


check = AddExists()



from __future__ import annotations

from checkov.github.repository_creation_is_limited import RepositoryCreationIsLimited


class GithubPublicRepositoryCreationIsLimited(RepositoryCreationIsLimited):
    def __init__(self) -> None:
        name = "Ensure public repository creation is limited to specific members"
        id = "CKV_GITHUB_21"
        super().__init__(id=id, name=name)

    def get_evaluated_keys(self) -> list[str]:
        return ["members_can_create_public_repositories"]


check = GithubPublicRepositoryCreationIsLimited()



from __future__ import annotations

import argparse
import itertools
import json
import logging
import os
import re

from collections import defaultdict
from collections.abc import Iterable
from pathlib import Path
from typing import List, Dict, Any, Optional, cast, TYPE_CHECKING, Type

from typing_extensions import Literal

from checkov.common.bridgecrew.code_categories import CodeCategoryMapping
from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    integration as metadata_integration
from checkov.common.bridgecrew.integration_features.features.repo_config_integration import \
    integration as repo_config_integration
from checkov.common.bridgecrew.integration_features.features.licensing_integration import \
    integration as licensing_integration
from checkov.common.bridgecrew.integration_features.integration_feature_registry import integration_feature_registry
from checkov.common.bridgecrew.platform_errors import ModuleNotEnabledError
from checkov.common.bridgecrew.severities import Severities
from checkov.common.images.image_referencer import ImageReferencer
from checkov.common.output.csv import CSVSBOM
from checkov.common.output.cyclonedx import CycloneDX
from checkov.common.output.report import Report, merge_reports
from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.typing import _ExitCodeThresholds, _BaseRunner
from checkov.common.util import data_structures_utils
from checkov.common.util.banner import tool as tool_name
from checkov.common.util.json_utils import CustomJSONEncoder
from checkov.common.util.secrets_omitter import SecretsOmitter
from checkov.common.util.type_forcers import convert_csv_string_arg_to_list, force_list
from checkov.sca_image.runner import Runner as image_runner
from checkov.terraform.context_parsers.registry import parser_registry
from checkov.terraform.parser import Parser
from checkov.terraform.runner import Runner as tf_runner

if TYPE_CHECKING:
    from checkov.common.output.baseline import Baseline
    from checkov.common.runners.base_runner import BaseRunner  # noqa
    from checkov.runner_filter import RunnerFilter

CONSOLE_OUTPUT = "console"
CHECK_BLOCK_TYPES = frozenset(["resource", "data", "provider", "module"])
CYCLONEDX_OUTPUTS = ("cyclonedx", "cyclonedx_json")
OUTPUT_CHOICES = ["cli", "cyclonedx", "cyclonedx_json", "json", "junitxml", "github_failed_only", "sarif", "csv"]
SUMMARY_POSITIONS = frozenset(['top', 'bottom'])
OUTPUT_DELIMITER = "\n--- OUTPUT DELIMITER ---\n"


class RunnerRegistry:
    def __init__(self, banner: str, runner_filter: RunnerFilter, *runners: _BaseRunner,
                 secrets_omitter_class: Type[SecretsOmitter] = SecretsOmitter) -> None:
        self.logger = logging.getLogger(__name__)
        self.runner_filter = runner_filter
        self.runners = list(runners)
        self.banner = banner
        self.scan_reports: list[Report] = []
        self.image_referencing_runners = self._get_image_referencing_runners()
        self.filter_runner_framework()
        self.tool = tool_name
        self._check_type_to_report_map: dict[str, Report] = {}  # used for finding reports with the same check type
        self.licensing_integration = licensing_integration  # can be maniuplated by unit tests
        self.secrets_omitter_class = secrets_omitter_class
        for runner in runners:
            if isinstance(runner, image_runner):
                runner.image_referencers = self.image_referencing_runners

    def run(
            self,
            root_folder: Optional[str] = None,
            external_checks_dir: Optional[List[str]] = None,
            files: Optional[List[str]] = None,
            collect_skip_comments: bool = True,
            repo_root_for_plan_enrichment: list[str | Path] | None = None,
    ) -> list[Report]:
        if len(self.runners) == 1:
            runner_check_type = self.runners[0].check_type
            if self.licensing_integration.is_runner_valid(runner_check_type):
                reports: Iterable[Report | list[Report]] = [
                    self.runners[0].run(root_folder, external_checks_dir=external_checks_dir, files=files,
                                        runner_filter=self.runner_filter,
                                        collect_skip_comments=collect_skip_comments)]
            else:
                # This is the only runner, so raise a clear indication of failure
                raise ModuleNotEnabledError(f'The framework "{runner_check_type}" is part of the "{self.licensing_integration.get_subscription_for_runner(runner_check_type).name}" module, which is not enabled in the platform')
        else:
            def _parallel_run(runner: _BaseRunner) -> Report | list[Report]:
                report = runner.run(
                    root_folder=root_folder,
                    external_checks_dir=external_checks_dir,
                    files=files,
                    runner_filter=self.runner_filter,
                    collect_skip_comments=collect_skip_comments,
                )
                if report is None:
                    # this only happens, when an uncaught exception inside the runner occurs
                    logging.error(f"Failed to create report for {runner.check_type} framework")
                    report = Report(check_type=runner.check_type)

                return report

            valid_runners = []
            invalid_runners = []

            for runner in self.runners:
                if self.licensing_integration.is_runner_valid(runner.check_type):
                    valid_runners.append(runner)
                else:
                    invalid_runners.append(runner)

            # if all runners are disabled (most likely to occur if the user specified --framework for only disabled runners)
            # then raise a clear error
            # if some frameworks are disabled and the user used --framework, log a warning so they see it
            # if some frameworks are disabled and the user did not use --framework, then log at a lower level so that we have it for troubleshooting
            frameworks_specified = self.runner_filter.framework and 'all' not in self.runner_filter.framework
            if not valid_runners:
                runners_categories = os.linesep.join([f'{runner.check_type}: {self.licensing_integration.get_subscription_for_runner(runner.check_type).name}' for runner in invalid_runners])
                error_message = f'All the frameworks are disabled because they are not enabled in the platform. ' \
                                f'You must subscribe to one or more of the categories below to get results for these frameworks.{os.linesep}{runners_categories}'
                logging.error(error_message)
                raise ModuleNotEnabledError(error_message)
            elif invalid_runners:
                level = logging.WARNING if frameworks_specified else logging.INFO
                for runner in invalid_runners:
                    logging.log(level, f'The framework "{runner.check_type}" is part of the "{self.licensing_integration.get_subscription_for_runner(runner.check_type).name}" module, which is not enabled in the platform')

            reports = [r for r in parallel_runner.run_function(func=_parallel_run, items=valid_runners, group_size=1) if r]

        merged_reports = self._merge_reports(reports)

        if bc_integration.bc_api_key:
            self.secrets_omitter_class(merged_reports).omit()

        for scan_report in merged_reports:
            self._handle_report(scan_report, repo_root_for_plan_enrichment)
        return self.scan_reports

    def _merge_reports(self, reports: Iterable[Report | list[Report]]) -> list[Report]:
        """Merges reports with the same check_type"""

        merged_reports = []

        for report in reports:
            if report is None:
                # this only happens, when an uncaught exception occurs
                continue

            sub_reports: list[Report] = force_list(report)
            for sub_report in sub_reports:
                if sub_report.check_type in self._check_type_to_report_map:
                    merge_reports(self._check_type_to_report_map[sub_report.check_type], sub_report)
                else:
                    self._check_type_to_report_map[sub_report.check_type] = sub_report
                    merged_reports.append(sub_report)

        return merged_reports

    def _handle_report(self, scan_report: Report, repo_root_for_plan_enrichment: list[str | Path] | None) -> None:
        integration_feature_registry.run_post_runner(scan_report)
        if metadata_integration.check_metadata:
            RunnerRegistry.enrich_report_with_guidelines(scan_report)
        if repo_root_for_plan_enrichment:
            enriched_resources = RunnerRegistry.get_enriched_resources(
                repo_roots=repo_root_for_plan_enrichment,
                download_external_modules=self.runner_filter.download_external_modules,
            )
            scan_report = Report("terraform_plan").enrich_plan_report(scan_report, enriched_resources)
            scan_report = Report("terraform_plan").handle_skipped_checks(scan_report, enriched_resources)
        self.scan_reports.append(scan_report)

    def save_output_to_file(self, file_name: str, data: str, data_format: str) -> None:
        try:
            file_path = Path(file_name)
            file_path.parent.mkdir(parents=True, exist_ok=True)
            file_path.write_text(data)
            logging.info(f"\nWrote output in {data_format} format to the file '{file_name}')")
        except EnvironmentError:
            logging.error(f"\nAn error occurred while writing {data_format} results to file: {file_name}",
                          exc_info=True)

    @staticmethod
    def get_fail_thresholds(config: argparse.Namespace, report_type: str) -> _ExitCodeThresholds:

        soft_fail = config.soft_fail

        soft_fail_on_checks = []
        soft_fail_threshold = None
        # soft fail on the highest severity threshold in the list
        for val in convert_csv_string_arg_to_list(config.soft_fail_on):
            if val.upper() in Severities:
                val = val.upper()
                if not soft_fail_threshold or Severities[val].level > soft_fail_threshold.level:
                    soft_fail_threshold = Severities[val]
            else:
                soft_fail_on_checks.append(val)

        logging.debug(f'Soft fail severity threshold: {soft_fail_threshold.level if soft_fail_threshold else None}')
        logging.debug(f'Soft fail checks: {soft_fail_on_checks}')

        hard_fail_on_checks = []
        hard_fail_threshold = None
        # hard fail on the lowest threshold in the list
        for val in convert_csv_string_arg_to_list(config.hard_fail_on):
            if val.upper() in Severities:
                val = val.upper()
                if not hard_fail_threshold or Severities[val].level < hard_fail_threshold.level:
                    hard_fail_threshold = Severities[val]
            else:
                hard_fail_on_checks.append(val)

        logging.debug(f'Hard fail severity threshold: {hard_fail_threshold.level if hard_fail_threshold else None}')
        logging.debug(f'Hard fail checks: {hard_fail_on_checks}')

        if not config.use_enforcement_rules:
            logging.debug('Use enforcement rules is FALSE')
        elif not soft_fail:
            code_category_type = CodeCategoryMapping[report_type]
            enf_rule = repo_config_integration.code_category_configs.get(code_category_type)

            if enf_rule:
                logging.debug('Use enforcement rules is TRUE')

                # if there is a severity in either the soft-fail-on list or hard-fail-on list, then we will ignore enforcement rules
                # if the lists only contain check IDs, then we will merge them with the enforcement rule value
                if soft_fail_threshold or hard_fail_threshold:
                    logging.debug('Soft or hard fail threshold is set; ignoring enforcement rules')
                else:
                    hard_fail_threshold = enf_rule.hard_fail_threshold
                    soft_fail = enf_rule.is_global_soft_fail()
                    logging.debug(f'Using enforcement rule hard fail threshold for this report: {hard_fail_threshold.name}')
            else:
                logging.debug(f'Use enforcement rules is TRUE, but did not find an enforcement rule for report type {report_type}, so falling back to CLI args')
        else:
            logging.debug('Soft fail was true; ignoring enforcement rules')

        return {
            'soft_fail': soft_fail,
            'soft_fail_checks': soft_fail_on_checks,
            'soft_fail_threshold': soft_fail_threshold,
            'hard_fail_checks': hard_fail_on_checks,
            'hard_fail_threshold': hard_fail_threshold
        }

    def print_reports(
            self,
            scan_reports: List[Report],
            config: argparse.Namespace,
            url: Optional[str] = None,
            created_baseline_path: Optional[str] = None,
            baseline: Optional[Baseline] = None,
    ) -> Literal[0, 1]:
        output_formats: "dict[str, str]" = {}

        if config.output_file_path and "," in config.output_file_path:
            output_paths = config.output_file_path.split(",")
            for idx, output_format in enumerate(config.output):
                output_formats[output_format] = output_paths[idx]
        else:
            output_formats = {output_format: CONSOLE_OUTPUT for output_format in config.output}

        exit_codes = []
        cli_reports = []
        report_jsons = []
        sarif_reports = []
        junit_reports = []
        github_reports = []
        cyclonedx_reports = []
        csv_sbom_report = CSVSBOM()

        try:
            if config.skip_resources_without_violations:
                for report in scan_reports:
                    report.extra_resources = set()
        except AttributeError:
            # config attribute wasn't set, defaults to False and print extra resources to report
            pass

        data_outputs: dict[str, str] = defaultdict(str)
        for report in scan_reports:
            if not report.is_empty():
                if "json" in config.output:
                    report_jsons.append(report.get_dict(is_quiet=config.quiet, url=url))
                if "junitxml" in config.output:
                    junit_reports.append(report)
                if "github_failed_only" in config.output:
                    github_reports.append(report.print_failed_github_md(use_bc_ids=config.output_bc_ids))
                if "sarif" in config.output:
                    sarif_reports.append(report)
                if "cli" in config.output:
                    cli_reports.append(report)
                if any(cyclonedx in config.output for cyclonedx in CYCLONEDX_OUTPUTS):
                    cyclonedx_reports.append(report)
                if "csv" in config.output:
                    git_org = ""
                    git_repository = ""
                    if 'repo_id' in config and config.repo_id is not None:
                        git_org, git_repository = config.repo_id.split('/')
                    csv_sbom_report.add_report(report=report, git_org=git_org, git_repository=git_repository)
            logging.debug(f'Getting exit code for report {report.check_type}')
            exit_code_thresholds = self.get_fail_thresholds(config, report.check_type)
            exit_codes.append(report.get_exit_code(exit_code_thresholds))

        if "github_failed_only" in config.output:
            github_output = "".join(github_reports)

            self._print_to_console(
                output_formats=output_formats,
                output_format="github_failed_only",
                output=github_output,
            )

            data_outputs["github_failed_only"] = github_output
        if "cli" in config.output:
            if not config.quiet:
                print(f"{self.banner}\n")

            cli_output = ''
            for report in cli_reports:
                cli_output += report.print_console(
                    is_quiet=config.quiet,
                    is_compact=config.compact,
                    created_baseline_path=created_baseline_path,
                    baseline=baseline,
                    use_bc_ids=config.output_bc_ids,
                    summary_position=config.summary_position
                )

            self._print_to_console(
                output_formats=output_formats,
                output_format="cli",
                output=cli_output,
                url=url,
            )

            # Remove colors from the cli output
            ansi_escape = re.compile(r'(?:\x1B[@-_]|[\x80-\x9F])[0–9:;<=>?]*[ -/]*[@-~]')
            data_outputs['cli'] = ansi_escape.sub('', cli_output)
        if "sarif" in config.output:
            master_report = Report("merged")

            output_format = output_formats["sarif"]
            if "cli" not in config.output and output_format == CONSOLE_OUTPUT:
                print(self.banner)

            for report in sarif_reports:
                if "cli" not in config.output and output_format == CONSOLE_OUTPUT:
                    print(report.print_console(
                        is_quiet=config.quiet,
                        is_compact=config.compact,
                        created_baseline_path=created_baseline_path,
                        baseline=baseline,
                        use_bc_ids=config.output_bc_ids,
                        summary_position=config.summary_position
                    ))
                master_report.failed_checks += report.failed_checks
                master_report.skipped_checks += report.skipped_checks

            if output_format == CONSOLE_OUTPUT:
                # don't write to file, if an explicit file path was set
                master_report.write_sarif_output(self.tool)

            if output_format == CONSOLE_OUTPUT:
                del output_formats["sarif"]

                if "cli" not in config.output and url:
                    print("More details: {}".format(url))
                if CONSOLE_OUTPUT in output_formats.values():
                    print(OUTPUT_DELIMITER)

            data_outputs["sarif"] = json.dumps(master_report.get_sarif_json(self.tool), cls=CustomJSONEncoder)
        if "json" in config.output:
            if config.compact and report_jsons:
                self.strip_code_blocks_from_json(report_jsons)

            report_json_output: "list[dict[str, Any]] | dict[str, Any]" = report_jsons
            if not report_jsons:
                report_json_output = Report("").get_summary()
            elif len(report_jsons) == 1:
                report_json_output = report_jsons[0]

            json_output = json.dumps(report_json_output, indent=4, cls=CustomJSONEncoder)

            self._print_to_console(
                output_formats=output_formats,
                output_format="json",
                output=json_output,
            )

            data_outputs["json"] = json.dumps(report_json_output, cls=CustomJSONEncoder)
        if "junitxml" in config.output:
            properties = Report.create_test_suite_properties_block(config)

            if junit_reports:
                test_suites = [
                    report.get_test_suite(properties=properties, use_bc_ids=config.output_bc_ids)
                    for report in junit_reports
                ]
            else:
                test_suites = [Report("").get_test_suite(properties=properties)]

            junit_output = Report.get_junit_xml_string(test_suites)

            self._print_to_console(
                output_formats=output_formats,
                output_format="junitxml",
                output=junit_output,
            )

            data_outputs['junitxml'] = junit_output
        if any(cyclonedx in config.output for cyclonedx in CYCLONEDX_OUTPUTS):
            cyclonedx = CycloneDX(repo_id=metadata_integration.bc_integration.repo_id, reports=cyclonedx_reports)

            for cyclonedx_format in CYCLONEDX_OUTPUTS:
                if cyclonedx_format not in config.output:
                    # only the XML or JSON format was chosen
                    continue

                if cyclonedx_format == "cyclonedx":
                    cyclonedx_output = cyclonedx.get_xml_output()
                elif cyclonedx_format == "cyclonedx_json":
                    cyclonedx_output = cyclonedx.get_json_output()
                else:
                    # this shouldn't happen
                    logging.error(f"CycloneDX output format '{cyclonedx_format}' not supported")
                    continue

                self._print_to_console(
                    output_formats=output_formats,
                    output_format=cyclonedx_format,
                    output=cyclonedx_output,
                )

                data_outputs[cyclonedx_format] = cyclonedx_output
        if "csv" in config.output:
            is_api_key = False
            if 'bc_api_key' in config and config.bc_api_key is not None:
                is_api_key = True
            csv_sbom_report.persist_report(is_api_key=is_api_key, output_path=config.output_file_path)

        # Save output to file
        file_names = {
            'cli': 'results_cli.txt',
            'github_failed_only': 'results_github_failed_only.md',
            'sarif': 'results_sarif.sarif',
            'json': 'results_json.json',
            'junitxml': 'results_junitxml.xml',
            'cyclonedx': 'results_cyclonedx.xml',
            'cyclonedx_json': 'results_cyclonedx.json',
        }

        if config.output_file_path:
            if output_formats:
                for output_format, output_path in output_formats.items():
                    self.save_output_to_file(
                        file_name=output_path,
                        data=data_outputs[output_format],
                        data_format=output_format,
                    )
            else:
                for output in config.output:
                    if output in file_names:
                        self.save_output_to_file(
                            file_name=f'{config.output_file_path}/{file_names[output]}',
                            data=data_outputs[output],
                            data_format=output,
                        )
        exit_code = 1 if 1 in exit_codes else 0
        return cast(Literal[0, 1], exit_code)

    def _print_to_console(self, output_formats: dict[str, str], output_format: str, output: str, url: str | None = None) -> None:
        """Prints the output to console, if needed"""

        output_dest = output_formats[output_format]
        if output_dest == CONSOLE_OUTPUT:
            del output_formats[output_format]

            print(output)
            if url:
                print(f"More details: {url}")
            if CONSOLE_OUTPUT in output_formats.values():
                print(OUTPUT_DELIMITER)

    def print_iac_bom_reports(self, output_path: str,
                              scan_reports: list[Report],
                              output_types: list[str],
                              account_id: str) -> dict[str, str]:

        output_files = {
            'cyclonedx': 'results_cyclonedx.xml',
            'csv': 'results_iac.csv'
        }

        # create cyclonedx report
        if 'cyclonedx' in output_types:
            cyclonedx_output_path = output_files['cyclonedx']
            cyclonedx = CycloneDX(reports=scan_reports,
                                  repo_id=metadata_integration.bc_integration.repo_id,
                                  export_iac_only=True)
            cyclonedx_output = cyclonedx.get_xml_output()
            self.save_output_to_file(file_name=os.path.join(output_path, cyclonedx_output_path),
                                     data=cyclonedx_output,
                                     data_format="cyclonedx")

        # create csv report
        if 'csv' in output_types:
            csv_sbom_report = CSVSBOM()
            for report in scan_reports:
                if not report.is_empty():
                    git_org, git_repository = self.extract_git_info_from_account_id(account_id)
                    csv_sbom_report.add_report(report=report, git_org=git_org, git_repository=git_repository)
            csv_sbom_report.persist_report_iac(file_name=output_files['csv'], output_path=output_path)

        return {key: os.path.join(output_path, value) for key, value in output_files.items()}

    def filter_runner_framework(self) -> None:
        if not self.runner_filter:
            return
        if not self.runner_filter.framework:
            return
        if "all" in self.runner_filter.framework:
            return
        self.runners = [runner for runner in self.runners if runner.check_type in self.runner_filter.framework]

    def filter_runners_for_files(self, files: List[str]) -> None:
        if not files:
            return

        self.runners = [runner for runner in self.runners if any(runner.should_scan_file(file) for file in files)]
        logging.debug(f'Filtered runners based on file type(s). Result: {[r.check_type for r in self.runners]}')

    def remove_runner(self, runner: _BaseRunner) -> None:
        if runner in self.runners:
            self.runners.remove(runner)

    @staticmethod
    def enrich_report_with_guidelines(scan_report: Report) -> None:
        for record in itertools.chain(scan_report.failed_checks, scan_report.passed_checks, scan_report.skipped_checks):
            guideline = metadata_integration.get_guideline(record.check_id)
            if guideline:
                record.set_guideline(guideline)

    @staticmethod
    def get_enriched_resources(
        repo_roots: list[str | Path], download_external_modules: bool
    ) -> dict[str, dict[str, Any]]:
        repo_definitions = {}
        for repo_root in repo_roots:
            tf_definitions: dict[str, Any] = {}
            parsing_errors: dict[str, Exception] = {}
            Parser().parse_directory(
                directory=repo_root,  # assume plan file is in the repo-root
                out_definitions=tf_definitions,
                out_parsing_errors=parsing_errors,
                download_external_modules=download_external_modules,
            )
            repo_definitions[repo_root] = {'tf_definitions': tf_definitions, 'parsing_errors': parsing_errors}

        enriched_resources = {}
        for repo_root, parse_results in repo_definitions.items():
            for full_file_path, definition in parse_results['tf_definitions'].items():
                definitions_context = parser_registry.enrich_definitions_context((full_file_path, definition))
                abs_scanned_file, _ = tf_runner._strip_module_referrer(full_file_path)
                scanned_file = os.path.relpath(abs_scanned_file, repo_root)
                for block_type, block_value in definition.items():
                    if block_type in CHECK_BLOCK_TYPES:
                        for entity in block_value:
                            context_parser = parser_registry.context_parsers[block_type]
                            definition_path = context_parser.get_entity_context_path(entity)
                            entity_id = ".".join(definition_path)
                            entity_context_path = [block_type] + definition_path
                            entity_context = data_structures_utils.get_inner_dict(
                                definitions_context[full_file_path], entity_context_path
                            )
                            entity_lines_range = [
                                entity_context.get("start_line"),
                                entity_context.get("end_line"),
                            ]
                            entity_code_lines = entity_context.get("code_lines")
                            skipped_checks = entity_context.get("skipped_checks")
                            enriched_resources[entity_id] = {
                                "entity_code_lines": entity_code_lines,
                                "entity_lines_range": entity_lines_range,
                                "scanned_file": scanned_file,
                                "skipped_checks": skipped_checks,
                            }
        return enriched_resources

    def _get_image_referencing_runners(self) -> set[ImageReferencer]:
        image_referencing_runners: set[ImageReferencer] = set()
        for runner in self.runners:
            if issubclass(runner.__class__, ImageReferencer):
                image_referencing_runners.add(cast(ImageReferencer, runner))

        return image_referencing_runners

    @staticmethod
    def strip_code_blocks_from_json(report_jsons: List[Dict[str, Any]]) -> None:
        for report in report_jsons:
            results = report.get('results', {})
            for result in results.values():
                for result_dict in result:
                    result_dict["code_block"] = None
                    result_dict["connected_node"] = None

    @staticmethod
    def extract_git_info_from_account_id(account_id: str) -> tuple[str, str]:
        if '/' in account_id:
            account_id_list = account_id.split('/')
            git_org = '/'.join(account_id_list[0:-1])
            git_repository = account_id_list[-1]
        else:
            git_org, git_repository = "", ""

        return git_org, git_repository



import os

from checkov.terraform.module_loading.loaders.git_loader import GenericGitLoader
from checkov.terraform.module_loading.module_params import ModuleParams


class GithubAccessTokenLoader(GenericGitLoader):
    def discover(self, module_params: ModuleParams):
        self.module_source_prefix = "github.com"
        module_params.username = "x-access-token"
        module_params.token = os.getenv('GITHUB_PAT', '')

    def _is_matching_loader(self, module_params: ModuleParams) -> bool:
        if module_params.token:
            # if GITHUB_PAT is set and previous loaders failed, convert source (github ssh or github http or generic git)
            # to use the token in generic format git::https://x-access-token:<token>@github.com/org/repo.git
            self.logger.debug("GITHUB_PAT found. Attempting to clone module using HTTP basic authentication.")
            # if module_source = github.com/org/repo
            if module_params.module_source.startswith(self.module_source_prefix):
                module_params.module_source = f"git::https://{module_params.username}:{module_params.token}@{module_params.module_source}"
                return True
            # if module_source = git::https://github.com/org/repo.git
            if module_params.module_source.startswith(f"git::https://{self.module_source_prefix}"):
                module_params.module_source = f"git::https://{module_params.username}:{module_params.token}@{module_params.module_source.split('git::https://')[1]}"
                return True
            # if module_source = git@github.com:org/repo.git
            if module_params.module_source.startswith(f"git@{self.module_source_prefix}:"):
                module_params.module_source = f"git::https://{module_params.username}:{module_params.token}@{module_params.module_source.split('git@')[1].replace(':', '/')}"
                return True
            # if module_source = git::ssh://git@github.com/org/repo.git
            if module_params.module_source.startswith(f"git::ssh://git@{self.module_source_prefix}"):
                module_params.module_source = f"git::https://{module_params.username}:{module_params.token}@{module_params.module_source.split('git::ssh://git@')[1]}"
                return True

        return False


loader = GithubAccessTokenLoader()



import logging
from typing import Dict

from checkov.cloudformation.parser.cfn_yaml import ContentType
from checkov.cloudformation.parser import cfn_yaml

LOGGER = logging.getLogger(__name__)


def parse(filename, out_parsing_errors: Dict[str, str]):
    """
        Decode filename into an object
    """
    logging.debug(f"[tf_plan] - Parsing file {filename}")

    try:
        template, template_lines = cfn_yaml.load(filename, ContentType.TFPLAN)
    except Exception as e:
        logging.debug(f"[tf_plan] - Failed to parse file {filename}", exc_info=True)
        out_parsing_errors[filename] = str(e)
        return None, None

    if (
        template is not None
        and isinstance(template, dict)
        and 'terraform_version' in template
        and 'planned_values' in template
    ):
        logging.debug(f"[tf_plan] - Successfully parsed file {filename}")

        return template, template_lines

    logging.debug(f"[tf_plan] - Missing required fields in file {filename}")
    return None, None



from __future__ import annotations

import logging
from typing import Set, Any, Generator, Pattern, Optional, Dict, Tuple, List, TYPE_CHECKING, cast

from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import \
    integration as metadata_integration
import yaml
from detect_secrets.constants import VerifiedResult
from detect_secrets.core.potential_secret import PotentialSecret
from detect_secrets.plugins.base import RegexBasedDetector
from detect_secrets.util.inject import call_function_with_arguments
import re

from checkov.common.bridgecrew.platform_integration import bc_integration

MIN_CHARACTERS = 5
MAX_CHARACTERS = 100

if TYPE_CHECKING:
    from detect_secrets.util.code_snippet import CodeSnippet


def load_detectors() -> list[dict[str, Any]]:
    detectors: List[dict[str, Any]] = []
    try:
        customer_run_config_response = bc_integration.customer_run_config_response
        policies_list: List[dict[str, Any]] = []
        if customer_run_config_response:
            policies_list = customer_run_config_response.get('secretsPolicies', [])
    except Exception as e:
        logging.error(f"Failed to get detectors from customer_run_config_response, error: {e}")
        return []

    if policies_list:
        detectors = modify_secrets_policy_to_detectors(policies_list)
    if detectors:
        logging.info(f"Successfully loaded {len(detectors)} detectors from bc_integration")
    return detectors


def modify_secrets_policy_to_detectors(policies_list: List[dict[str, Any]]) -> List[dict[str, Any]]:
    secrets_list = transforms_policies_to_detectors_list(policies_list)
    logging.info(f"(modify_secrets_policy_to_detectors) secrets_list = {secrets_list}")
    return secrets_list


def transforms_policies_to_detectors_list(custom_secrets: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
    custom_detectors: List[Dict[str, Any]] = []
    for secret_policy in custom_secrets:
        not_parsed = True
        code = secret_policy['code']
        if code:
            code_dict = yaml.safe_load(secret_policy['code'])
            if 'definition' in code_dict:
                if 'value' in code_dict['definition']:
                    not_parsed = False
                    check_id = secret_policy['checkovCheckId'] if secret_policy['checkovCheckId'] else \
                        secret_policy['incidentId']
                    if type(code_dict['definition']['value']) is str:
                        code_dict['definition']['value'] = [code_dict['definition']['value']]
                    for regex in code_dict['definition']['value']:
                        custom_detectors.append({'Name': secret_policy['title'],
                                                 'Check_ID': check_id,
                                                 'Regex': regex,
                                                 'isCustom': secret_policy['isCustom']})
                        if secret_policy['isCustom']:
                            metadata_integration.check_metadata[check_id] = {'id': check_id}
        if not_parsed:
            logging.info(f"policy : {secret_policy} could not be parsed")
    return custom_detectors


class CustomRegexDetector(RegexBasedDetector):
    secret_type = "Regex Detector"  # noqa: CCE003 # nosec
    denylist: Set[Pattern[str]] = set()  # noqa: CCE003

    def __init__(self) -> None:
        self.regex_to_metadata: dict[str, dict[str, Any]] = dict()
        self.denylist = set()
        detectors = load_detectors()

        for detector in detectors:
            self.denylist.add(re.compile(r'{}'.format(detector["Regex"])))
            self.regex_to_metadata[detector["Regex"]] = detector

    def analyze_line(
            self,
            filename: str,
            line: str,
            line_number: int = 0,
            context: Optional[CodeSnippet] = None,
            raw_context: Optional[CodeSnippet] = None,
            **kwargs: Any
    ) -> Set[PotentialSecret]:
        """This examines a line and finds all possible secret values in it."""
        output: Set[PotentialSecret] = set()
        for match, regex in self.analyze_string(line, **kwargs):
            try:
                verified_result = call_function_with_arguments(self.verify, secret=match, context=context)
                is_verified = True if verified_result == VerifiedResult.VERIFIED_TRUE else False
            except Exception:
                is_verified = False
            regex_data = self.regex_to_metadata[regex.pattern]
            ps = PotentialSecret(type=regex_data["Name"], filename=filename, secret=match,
                                 line_number=line_number, is_verified=is_verified)
            ps.check_id = self.regex_to_metadata[regex.pattern]["Check_ID"]  # type:ignore[attr-defined]
            if len(cast(str, ps.secret_value)) in range(MIN_CHARACTERS, MAX_CHARACTERS) or not regex_data['isCustom']:
                output.add(ps)
            else:
                logging.info(
                    f'Finding for check {ps.check_id} are not 5-100 characters in length, was ignored')  # type: ignore

        return output

    def analyze_string(self, string: str, **kwargs: Optional[Dict[str, Any]]) -> Generator[Tuple[str, Pattern[str]], None, None]:  # type:ignore[override]
        for regex in self.denylist:
            for match in regex.findall(string):
                if isinstance(match, tuple):
                    for submatch in filter(bool, match):
                        # It might make sense to paste break after yielding
                        yield submatch, regex
                else:
                    yield match, regex



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.secrets import string_has_secrets, AWS, GENERAL
from checkov.common.util.type_forcers import force_list


class LambdaEnvironmentCredentials(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure no hard-coded secrets exist in lambda environment"
        id = "CKV_AWS_45"
        supported_resources = ["aws_lambda_function"]
        categories = [CheckCategories.SECRETS]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        environment = conf.get("environment", [])
        if environment and isinstance(environment[0], dict):
            self.evaluated_keys = ["environment"]

            variables = force_list(environment[0].get("variables", []))
            if variables and isinstance(variables[0], dict):
                self.evaluated_keys = ["environment/[0]/variables"]

                violated_envs = set()
                for key, values in variables[0].items():
                    # variables can be a string, which in this case it points to a variable
                    for idx, value in enumerate([v for v in force_list(values) if isinstance(v, str)]):
                        if string_has_secrets(value, AWS, GENERAL):
                            conf[f'{self.id}_secret_{idx}'] = value
                            violated_envs.add(key)

                if violated_envs:
                    self.evaluated_keys = [f"environment/[0]/variables/[0]/{env_key}" for env_key in violated_envs]

                    return CheckResult.FAILED
        return CheckResult.PASSED


check = LambdaEnvironmentCredentials()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.util.type_forcers import force_list
import json
from typing import List


class S3AllowsAnyPrincipal(BaseResourceCheck):

    def __init__(self):
        name = "Ensure S3 bucket does not allow an action with any Principal"
        id = "CKV_AWS_70"
        supported_resources = ['aws_s3_bucket', 'aws_s3_bucket_policy']
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if 'policy' not in conf.keys() or not isinstance(conf['policy'][0], str):
            return CheckResult.PASSED
        try:
            policy_block = json.loads(conf['policy'][0])
            if 'Statement' in policy_block.keys():
                for statement in force_list(policy_block['Statement']):
                    if statement['Effect'] == 'Deny' or 'Principal' not in statement:
                        continue

                    principal = statement['Principal']
                    if principal == '*':
                        return CheckResult.FAILED
                    if 'AWS' in statement['Principal']:
                        # Can be a string or an array of strings
                        aws = statement['Principal']['AWS']
                        if (isinstance(aws, str) and aws == '*') or (isinstance(aws, list) and '*' in aws):
                            return CheckResult.FAILED
        except Exception:  # nosec
            pass
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['policy']


check = S3AllowsAnyPrincipal()



from __future__ import annotations

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from typing import Any


class LambdaFunctionLevelConcurrentExecutionLimit(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure that AWS Lambda function is configured for function-level concurrent execution limit"
        id = "CKV_AWS_115"
        supported_resources = ("aws_lambda_function",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_attribute_result=CheckResult.FAILED,
        )

    def get_inspected_key(self) -> str:
        return "reserved_concurrent_executions/[0]"

    def get_forbidden_values(self) -> list[Any]:
        return ["${-1}"]


check = LambdaFunctionLevelConcurrentExecutionLimit()



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Dict, List

from checkov.azure_pipelines.checks.registry import registry
from checkov.common.output.report import CheckType
from checkov.yaml_doc.runner import Runner as YamlRunner
from checkov.common.util.consts import START_LINE, END_LINE


if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry
    from collections.abc import Iterable


class Runner(YamlRunner):
    check_type = CheckType.AZURE_PIPELINES  # noqa: CCE003  # a static attribute

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if self.is_workflow_file(f):
            return super()._parse_file(f=f)

        return None

    def is_workflow_file(self, file_path: str) -> bool:
        return file_path.endswith(('azure-pipelines.yml', 'azure-pipelines.yaml'))

    def get_resource(self, file_path: str, key: str, supported_entities: Iterable[str],
                     start_line: int = -1, end_line: int = -1) -> str:
        if not self.definitions or not isinstance(self.definitions, dict):
            return key
        resource_name = generate_resource_key_recursive(start_line, end_line, self.definitions[file_path])
        return resource_name if resource_name else key


def generate_resource_key_recursive(start_line: int, end_line: int,
                                    file_conf: Dict[str, Any] | List[Dict[str, Any]], resource_key: str | None = None
                                    ) -> str | None:
    if not isinstance(file_conf, dict):
        return resource_key

    def _get_resource_from_code_block(block_to_inspect: dict[str, Any], inspected_key: str | None) -> str | None:
        if block_to_inspect[START_LINE] <= start_line <= end_line <= block_to_inspect[END_LINE]:
            block_name = block_to_inspect.get('displayName',
                                              block_to_inspect.get('name',
                                                                   block_to_inspect.get('job',
                                                                                        block_to_inspect.get('stage',
                                                                                                             False))))
            inspected_key = f'{inspected_key}({block_name})' if block_name else inspected_key
            if block_to_inspect[START_LINE] == start_line:
                return inspected_key
            return generate_resource_key_recursive(start_line, end_line, block_to_inspect, resource_key=inspected_key)
        return None

    for code_block_name, code_block in file_conf.items():
        if isinstance(code_block, dict):
            new_key = f'{resource_key}.{code_block_name}' if resource_key else code_block_name
            resource = _get_resource_from_code_block(code_block, new_key)
            if resource:
                return resource
        elif isinstance(code_block, list):
            for index, item in enumerate(code_block):
                if isinstance(item, dict):
                    resource_key_to_inspect = f'{resource_key}.{code_block_name}[{index}]' if resource_key else f'{code_block_name}[{index}]'
                    resource = _get_resource_from_code_block(item, resource_key_to_inspect)
                    if resource:
                        return resource
    return resource_key



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult
from checkov.github_actions.checks.base_github_action_check import BaseGithubActionsCheck
from checkov.yaml_doc.enums import BlockType


class SuspectCurlInScript(BaseGithubActionsCheck):
    def __init__(self) -> None:
        name = "Suspicious use of curl with secrets"
        id = "CKV_GHA_3"
        super().__init__(
            name=name,
            id=id,
            block_type=BlockType.ARRAY,
            supported_entities=('jobs', 'jobs.*.steps[]')
        )

    def scan_conf(self, conf: dict[str, Any]) -> tuple[CheckResult, dict[str, Any]]:
        if not isinstance(conf, dict):
            return CheckResult.UNKNOWN, conf
        run = conf.get("run", "")
        if "curl" in run:
            badstuff = ('curl', 'secret')
            lines = run.split("\n")
            for line in lines:
                if all(x in line for x in badstuff):
                    return CheckResult.FAILED, conf
        return CheckResult.PASSED, conf


check = SuspectCurlInScript()



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class LivenessProbe(BaseK8sContainerCheck):
    def __init__(self) -> None:
        name = "Liveness Probe Should be Configured"
        id = "CKV_K8S_8"
        # Location: container .livenessProbe
        # Don't check Job/CronJob
        supported_entities = [
            entity for entity in BaseK8sContainerCheck.SUPPORTED_ENTITIES if entity not in ("CronJob", "Job")
        ]
        # initContainers do not need Liveness Probes...
        supported_container_types = ["containers"]
        super().__init__(
            name=name, id=id, supported_entities=supported_entities, supported_container_types=supported_container_types
        )

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["livenessProbe"]
        if conf.get("livenessProbe"):
            return CheckResult.PASSED
        return CheckResult.FAILED


check = LivenessProbe()



from __future__ import annotations

import itertools
from typing import List, Optional, Dict, Any, Tuple, TYPE_CHECKING

from checkov.common.graph.checks_infra.enums import SolverType
from checkov.common.graph.checks_infra.solvers.base_solver import BaseSolver
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.checks_infra.solvers.complex_solvers.base_complex_solver import BaseComplexSolver
from checkov.common.checks_infra.solvers.connections_solvers.base_connection_solver import BaseConnectionSolver
from checkov.common.checks_infra.solvers.filter_solvers.base_filter_solver import BaseFilterSolver
from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes

if TYPE_CHECKING:
    from networkx import DiGraph


class ComplexConnectionSolver(BaseConnectionSolver):
    def __init__(self, solvers: Optional[List[BaseSolver]], operator: str) -> None:
        self.solver_type = SolverType.COMPLEX_CONNECTION
        self.solvers = solvers if solvers else []
        self.operator = operator

        resource_types = set()
        connected_resources_types = set()
        for sub_solver in self.solvers:
            if isinstance(sub_solver, BaseConnectionSolver):
                resource_types.update(sub_solver.resource_types)
                connected_resources_types.update(sub_solver.connected_resources_types)
            elif isinstance(sub_solver, BaseAttributeSolver):
                resource_types.update(sub_solver.resource_types)

        super().__init__(list(resource_types), list(connected_resources_types))

    @staticmethod
    def filter_duplicates(checks: List[Dict[str, Any]]) -> List[Dict[str, Any]]:
        return list({(check[CustomAttributes.ID], check[CustomAttributes.FILE_PATH]): check for check in checks}.values())

    def filter_results(
        self, passed: List[Dict[str, Any]], failed: List[Dict[str, Any]], unknown: List[Dict[str, Any]]
    ) -> Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        filter_solvers = [sub_solver for sub_solver in self.solvers if isinstance(sub_solver, BaseFilterSolver)]
        for sub_solver in filter_solvers:
            filter_pred = sub_solver._get_operation()
            passed = list(filter(filter_pred, passed))
            failed = list(filter(filter_pred, failed))
            unknown = list(filter(filter_pred, unknown))
        passed = self.filter_duplicates(passed)
        failed = self.filter_duplicates(failed)
        unknown = self.filter_duplicates(unknown)
        return passed, failed, unknown

    def get_sorted_connection_solvers(self) -> List[BaseConnectionSolver]:
        connection_solvers = [sub_solver for sub_solver in self.solvers if isinstance(sub_solver, BaseConnectionSolver)]
        filter_solvers = [sub_solver for sub_solver in self.solvers if isinstance(sub_solver, BaseFilterSolver)]

        resource_types_to_filter = []
        for filter_solver in filter_solvers:
            if filter_solver.attribute == "resource_type":
                resource_types_to_filter.extend(filter_solver.value)

        sorted_connection_solvers = []
        connection_solvers_with_filtered_resource_types = []
        for connection_solver in connection_solvers:
            if any(
                r in resource_types_to_filter
                for r in itertools.chain(connection_solver.resource_types, connection_solver.connected_resources_types)
            ):
                connection_solvers_with_filtered_resource_types.append(connection_solver)
            else:
                sorted_connection_solvers.append(connection_solver)

        sorted_connection_solvers.extend(connection_solvers_with_filtered_resource_types)
        return sorted_connection_solvers

    def run_attribute_solvers(self, graph_connector: DiGraph) -> \
            Tuple[List[Dict[str, Any]], List[Dict[str, Any]], List[Dict[str, Any]]]:
        attribute_solvers = [
            sub_solver
            for sub_solver in self.solvers
            if isinstance(sub_solver, (BaseAttributeSolver, BaseComplexSolver))
        ]
        passed_attributes, failed_attributes, unknown_attributes = [], [], []
        for attribute_solver in attribute_solvers:
            passed_solver, failed_solver, unknown_solver = attribute_solver.run(graph_connector)
            passed_attributes.extend(passed_solver)
            failed_attributes.extend(failed_solver)
            unknown_attributes.extend(unknown_solver)

        return passed_attributes, failed_attributes, unknown_attributes



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.bicep.image_referencer.provider.azure import AzureBicepProvider

if TYPE_CHECKING:
    from checkov.common.images.image_referencer import Image
    from networkx import DiGraph


class BicepImageReferencerManager:
    __slots__ = ("graph_connector",)

    def __init__(self, graph_connector: DiGraph) -> None:
        self.graph_connector = graph_connector

    def extract_images_from_resources(self) -> list[Image]:
        bicep_provider = AzureBicepProvider(graph_connector=self.graph_connector)

        images = bicep_provider.extract_images_from_resources()

        return images



from __future__ import annotations

import logging
import os
from collections.abc import Collection
from pathlib import Path
from typing import Any, TYPE_CHECKING

from checkov.common.runners.base_runner import filter_ignored_paths
from checkov.runner_filter import RunnerFilter
from checkov.bicep.parser import Parser

if TYPE_CHECKING:
    from pycep.typing import BicepJson


BICEP_POSSIBLE_ENDINGS = [".bicep"]
BICEP_START_LINE = "__start_line__"
BICEP_END_LINE = "__end_line__"


def get_scannable_file_paths(root_folder: str | Path | None = None, files: list[str] | None = None) -> set[Path]:
    """Finds Bicep files"""

    file_paths: set[Path] = set()

    if root_folder:
        root_path = Path(root_folder)
        file_paths = {file_path for file_path in root_path.rglob("*.bicep") if file_path.is_file()}
    if files:
        for file in files:
            if file.endswith(".bicep"):
                file_paths.add(Path(file))

    return file_paths


def clean_file_path(file_path: Path) -> Path:
    path_parts = [part for part in file_path.parts if part not in (".", "..")]

    return Path(*path_parts)


def get_folder_definitions(
    root_folder: str, excluded_paths: list[str] | None
) -> tuple[dict[Path, BicepJson], dict[Path, list[tuple[int, str]]], list[str]]:
    files_list: set[Path] = set()
    for root, d_names, f_names in os.walk(root_folder):
        filter_ignored_paths(root, d_names, excluded_paths)
        filter_ignored_paths(root, f_names, excluded_paths)
        for file in f_names:
            file_ending = os.path.splitext(file)[1]
            if file_ending in BICEP_POSSIBLE_ENDINGS:
                full_path = os.path.join(root, file)
                files_list.add(Path(full_path))
    parser = Parser()

    return parser.get_files_definitions(files_list)


def create_definitions(
        root_folder: str,
        files: "Collection[Path] | None" = None,
        runner_filter: RunnerFilter | None = None,
) -> tuple[dict[Path, BicepJson], dict[Path, list[tuple[int, str]]]]:
    definitions: dict[Path, BicepJson] = {}
    definitions_raw: dict[Path, list[tuple[int, str]]] = {}
    parsing_errors: list[str] = []
    runner_filter = runner_filter or RunnerFilter()

    if files:
        parser = Parser()
        definitions, definitions_raw, parsing_errors = parser.get_files_definitions(file_paths=files)

    if root_folder:
        definitions, definitions_raw, parsing_errors = get_folder_definitions(root_folder,
                                                                              runner_filter.excluded_paths)

    if parsing_errors:
        logging.warning(f"[bicep] found errors while parsing definitions: {parsing_errors}")

    return definitions, definitions_raw


def adjust_value(element_name: str, value: Any) -> Any:
    """Adjusts the value, if the 'element_name' references a nested key

    Ex:
    element_name = publicKey.keyData
    value = {"keyData": "key-data", "path": "path"}

    returns new_value = "key-data"
    """

    if "." in element_name and isinstance(value, dict):
        key_parts = element_name.split(".")
        new_value = value.get(key_parts[1])

        if new_value is None:
            # couldn't find key in in value object
            return None

        return adjust_value(".".join(key_parts[1:]), new_value)

    return value



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class AKSPoolTypeIsScaleSet(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Azure Kubernetes Cluster (AKS) nodes use scale sets"
        id = "CKV_AZURE_169"
        supported_resources = ("azurerm_kubernetes_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "default_node_pool/[0]/type"

    def get_forbidden_values(self) -> list[Any]:
        return ["AvailabilitySet"]


check = AKSPoolTypeIsScaleSet()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from typing import Any


class ActionTrailLogAllRegions(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Action Trail Logging for all regions"
        id = "CKV_ALI_4"
        supported_resources = ("alicloud_actiontrail_trail",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.PASSED,
        )

    def get_inspected_key(self) -> str:
        return "trail_region"

    def get_expected_value(self) -> Any:
        return "All"


check = ActionTrailLogAllRegions()



from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class DataflowPrivateJob(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Dataflow jobs are private"
        id = "CKV_GCP_94"
        supported_resources = ("google_dataflow_job",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "ip_configuration"

    # Possible values are "WORKER_IP_PUBLIC" or "WORKER_IP_PRIVATE"
    def get_expected_value(self) -> Any:
        return "WORKER_IP_PRIVATE"


check = DataflowPrivateJob()



from __future__ import annotations
from typing import Any

from checkov.common.images.image_referencer import Image
from checkov.common.util.consts import START_LINE, END_LINE


class GithubActionProvider:
    __slots__ = ("workflow_config", "file_path", "workflow_line_numbers")

    def __init__(self, workflow_config: dict[str, Any], file_path: str, workflow_line_numbers: list[tuple[int, str]]):
        self.workflow_config = workflow_config
        self.file_path = file_path
        self.workflow_line_numbers = workflow_line_numbers

    def generate_resource_key(self, start_line: int, end_line: int) -> str:
        """
        Generate resource key without the previous format of key (needed in get_resource)
        """
        jobs_dict: dict[str, Any] = self.workflow_config.get("jobs", {})
        for job_name, job in jobs_dict.items():
            if not isinstance(job, dict):
                continue

            if job[START_LINE] <= start_line <= end_line <= job[END_LINE]:
                return f'jobs({job_name})'

        return ''

    def extract_images_from_workflow(self) -> list[Image]:
        images: list[Image] = []

        if not isinstance(self.workflow_config, dict):
            # make type checking happy
            return images

        jobs = self.workflow_config.get("jobs", {})
        for job_object in jobs.values():
            if isinstance(job_object, dict):
                container = job_object.get("container", {})
                image = None
                start_line = 0
                end_line = 0

                if isinstance(container, dict):
                    image = container.get("image", "")
                    start_line, end_line = container.get(START_LINE, 0), container.get(END_LINE, 0)

                elif isinstance(container, str):
                    image = container
                    start_line = [line_number for line_number, line in self.workflow_line_numbers if image in line][0]
                    end_line = start_line + 1

                if image:
                    image_obj = Image(
                        file_path=self.file_path[self.file_path.index('/.github'):],
                        name=image,
                        start_line=start_line,
                        end_line=end_line,
                        related_resource_id=self.generate_resource_key(start_line, end_line)
                    )
                    images.append(image_obj)

        return images



from __future__ import annotations

import asyncio
import json
import logging
import os
import time
from collections.abc import Iterable, Sequence, Collection
from pathlib import Path
from typing import Dict, Any, List

import requests

from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.common.bridgecrew.platform_key import bridgecrew_dir
from checkov.common.bridgecrew.vulnerability_scanning.image_scanner import image_scanner, TWISTCLI_FILE_NAME
from checkov.common.bridgecrew.vulnerability_scanning.integrations.docker_image_scanning import \
    docker_image_scanning_integration
from checkov.common.util.file_utils import compress_file_gzip_base64, decompress_file_gzip_base64
from checkov.common.util.http_utils import request_wrapper

from checkov.common.util.tqdm_utils import ProgressBar

SLEEP_DURATION = 2
MAX_SLEEP_DURATION = 60


class Scanner:
    def __init__(self, pbar: ProgressBar | None = None, root_folder: str | Path | None = None) -> None:
        self._base_url = bc_integration.api_url
        if pbar:
            self.pbar = pbar
        else:
            self.pbar = ProgressBar('')
            self.pbar.turn_off_progress_bar()
        self.root_folder = root_folder

    def scan(self, input_paths: Collection[Path]) -> Sequence[dict[str, Any]]:
        self.pbar.initiate(len(input_paths))
        scan_results = asyncio.run(
            self.run_scan_multi(input_paths=input_paths)
        )
        self.pbar.close()
        return scan_results

    async def run_scan_multi(
            self,
            input_paths: "Iterable[Path]",
    ) -> "Sequence[Dict[str, Any]]":

        if os.getenv("PYCHARM_HOSTED") == "1":
            # PYCHARM_HOSTED env variable equals 1 when running via Pycharm.
            # it avoids us from crashing, which happens when using multiprocessing via Pycharm's debug-mode
            logging.warning("Running the scans in sequence for avoiding crashing when running via Pycharm")
            scan_results: list[dict[str, Any]] = []
            for input_path in input_paths:
                scan_results.append(await self.run_scan(input_path))
        else:
            scan_results = await asyncio.gather(*[self.run_scan(i) for i in input_paths])

        if any(scan_result.get("packages") is None for scan_result in scan_results):
            image_scanner.setup_twistcli()

            if os.getenv("PYCHARM_HOSTED") == "1":
                # PYCHARM_HOSTED env variable equals 1 when running via Pycharm.
                # it avoids us from crashing, which happens when using multiprocessing via Pycharm's debug-mode
                logging.warning("Running the scans in sequence for avoiding crashing when running via Pycharm")
                scan_results = [
                    await self.execute_twistcli_scan(input_path) if scan_results[idx].get("packages") is None else
                    scan_results[idx] for idx, input_path in enumerate(input_paths)
                ]
            else:
                input_paths_as_list: List[Path] = list(input_paths)  # create a list from a set ("Iterable")
                indices_to_fix: List[int] = [
                    idx
                    for idx in range(len(input_paths_as_list))
                    if scan_results[idx].get("packages") is None
                ]
                new_scan_results = await asyncio.gather(*[
                    self.execute_twistcli_scan(input_paths_as_list[idx]) for idx in indices_to_fix
                ])
                for idx in indices_to_fix:
                    res = new_scan_results.pop(0)
                    res['repository'] = str(input_paths_as_list[idx])
                    scan_results[idx] = res

        return scan_results

    async def run_scan(self, input_path: Path) -> dict[str, Any]:
        self.pbar.set_additional_data({'Current File Scanned': os.path.relpath(input_path, self.root_folder)})
        logging.info(f"Start to scan package file {input_path}")

        request_body = {
            "compressedFileBody": compress_file_gzip_base64(str(input_path)),
            "compressionMethod": "gzip",
            "fileName": input_path.name
        }

        response = request_wrapper(
            "POST", f"{self._base_url}/api/v1/vulnerabilities/scan",
            headers=bc_integration.get_default_headers("POST"),
            json=request_body,
            should_call_raise_for_status=True
        )

        response_json = response.json()

        if response_json["status"] == "already_exist":
            logging.info(f"result for {input_path} exists in the cache")
            return self.parse_api_result(input_path, response_json["outputData"])

        return self.run_scan_busy_wait(input_path, response_json['id'])

    def run_scan_busy_wait(self, input_path: Path, scan_id: str) -> dict[str, Any]:
        current_state = "Empty"
        desired_state = "Result"
        total_sleeping_time = 0
        response = requests.Response()

        while current_state != desired_state:
            response = request_wrapper(
                "GET", f"{self._base_url}/api/v1/vulnerabilities/scan-results/{scan_id}",
                headers=bc_integration.get_default_headers("GET")
            )
            response_json = response.json()
            current_state = response_json["outputType"]

            if current_state == "Error":
                logging.error(response_json["outputData"])
                return {}

            if total_sleeping_time > MAX_SLEEP_DURATION:
                logging.info(f"Timeout, slept for {total_sleeping_time}")
                return {}

            time.sleep(SLEEP_DURATION)
            total_sleeping_time += SLEEP_DURATION

        return self.parse_api_result(input_path, response.json()["outputData"])

    def parse_api_result(self, origin_file_path: Path, response: str) -> dict[str, Any]:
        raw_result: dict[str, Any] = json.loads(decompress_file_gzip_base64(response))
        raw_result['repository'] = str(origin_file_path)
        self.pbar.update()
        return raw_result

    async def execute_twistcli_scan(
            self,
            input_path: Path,
    ) -> Dict[str, Any]:
        output_path = Path(f'results-{input_path.name}.json')

        command = f"{Path(bridgecrew_dir) / TWISTCLI_FILE_NAME} coderepo scan --address {docker_image_scanning_integration.get_proxy_address()} --token {docker_image_scanning_integration.get_bc_api_key()} --details --output-file \"{output_path}\" {input_path}"
        process = await asyncio.create_subprocess_shell(
            command, stdout=asyncio.subprocess.PIPE, stderr=asyncio.subprocess.PIPE
        )

        stdout, stderr = await process.communicate()

        # log output for debugging
        logging.debug(stdout.decode())

        exit_code = await process.wait()

        if exit_code:
            logging.error(stderr.decode())
            return {}

        # read the report file
        scan_result: Dict[str, Any] = json.loads(output_path.read_text())
        output_path.unlink()
        return scan_result



from checkov.common.vcs.vcs_schema import VCSSchema


class GroupsSchema(VCSSchema):
    def __init__(self) -> None:
        schema = {
            "$schema": "http://json-schema.org/draft-04/schema#",
            "type": "array",
            "items": [
                {
                    "type": "object",
                    "properties": {
                        "id": {
                            "type": "integer"
                        },
                        "web_url": {
                            "type": "string"
                        },
                        "name": {
                            "type": "string"
                        },
                        "path": {
                            "type": "string"
                        },
                        "description": {
                            "type": "string"
                        },
                        "visibility": {
                            "type": "string"
                        },
                        "share_with_group_lock": {
                            "type": "boolean"
                        },
                        "require_two_factor_authentication": {
                            "type": "boolean"
                        },
                        "two_factor_grace_period": {
                            "type": "integer"
                        },
                        "project_creation_level": {
                            "type": "string"
                        },
                        "auto_devops_enabled": {
                            "type": "null"
                        },
                        "subgroup_creation_level": {
                            "type": "string"
                        },
                        "emails_disabled": {
                            "oneOf": [
                                {"type": "boolean"},
                                {"type": "null"}
                            ]
                        },
                        "mentions_disabled": {
                            "oneOf": [
                                {"type": "boolean"},
                                {"type": "null"}
                            ]
                        },
                        "lfs_enabled": {
                            "oneOf": [
                                {"type": "boolean"},
                                {"type": "null"}
                            ]
                        },
                        "default_branch_protection": {
                            "type": "integer"
                        },
                        "avatar_url": {
                            "oneOf": [
                                {"type": "string"},
                                {"type": "null"}
                            ]
                        },
                        "request_access_enabled": {
                            "type": "boolean"
                        },
                        "full_name": {
                            "type": "string"
                        },
                        "full_path": {
                            "type": "string"
                        },
                        "created_at": {
                            "type": "string"
                        },
                        "parent_id": {
                            "type": "null"
                        },
                        "ldap_cn": {
                            "type": "null"
                        },
                        "ldap_access": {
                            "type": "null"
                        }
                    },
                    "required": [
                        "id",
                        "web_url",
                        "name",
                        "path",
                        "description",
                        "visibility",
                        "share_with_group_lock",
                        "require_two_factor_authentication",
                        "two_factor_grace_period",
                        "project_creation_level",
                        "auto_devops_enabled",
                        "subgroup_creation_level",
                        "emails_disabled",
                        "mentions_disabled",
                        "lfs_enabled",
                        "default_branch_protection",
                        "avatar_url",
                        "request_access_enabled",
                        "full_name",
                        "full_path",
                        "created_at",
                        "parent_id",
                        "ldap_cn",
                        "ldap_access"
                    ]
                }
            ]
        }
        super().__init__(schema=schema)


schema = GroupsSchema()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.kubernetes.checks.resource.base_spec_check import BaseK8Check


class DefaultServiceAccountBinding(BaseK8Check):
    def __init__(self) -> None:
        # CIS-1.5 5.1.5
        # Check no role/clusterrole is bound to a default service account (to ensure not actively used)
        # Location: .subjects[]
        name = "Ensure that default service accounts are not actively used"
        id = "CKV_K8S_42"
        supported_kind = ("RoleBinding", "ClusterRoleBinding")
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_kind)

    def scan_spec_conf(self, conf: dict[str, Any]) -> CheckResult:
        if "subjects" in conf and isinstance(conf["subjects"], list):
            for subject in conf["subjects"]:
                if subject["kind"] == "ServiceAccount":
                    if subject["name"] == "default":
                        return CheckResult.FAILED
        return CheckResult.PASSED


check = DefaultServiceAccountBinding()



import os
import platform

import pytest
import time

from checkov.cloudformation.runner import Runner as cfn_runner
from checkov.common.runners.runner_registry import RunnerRegistry
from checkov.common.util.banner import banner
from checkov.kubernetes.runner import Runner as k8_runner
from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner as tf_runner

# Ensure repo_name is a cloned repository into performance_tests directory.
# Thresholds are in ms, and are set to the current maximum duration of checkov on the repository
performance_configurations = {
    'terraform': {
        'repo_name': 'terraform-aws-components',
        'threshold': {
            "Darwin": 18.0,
            "Linux": 12.0,
            "Windows": 14.0,
        }
    },
    'cloudformation': {
        'repo_name': 'aws-cloudformation-templates',
        'threshold': {
            "Darwin": 350.0,
            "Linux": 250.0,
            "Windows": 300.0,
        }
    },
    'kubernetes': {
        'repo_name': 'kubernetes-yaml-templates',
        'threshold': {
            "Darwin": 550.0,
            "Linux": 300.0,
            "Windows": 500.0,
        }
    }
}

DEVIATION_PERCENT = 10
SYSTEM_NAME = platform.system()


@pytest.mark.benchmark(
    group="terraform-performance-tests",
    disable_gc=True,
    min_time=0.1,
    max_time=0.5,
    min_rounds=10,
    timer=time.time,
    warmup=False,
)
def test_terraform_performance(benchmark):
    repo_name = performance_configurations['terraform']['repo_name']
    repo_threshold = performance_configurations['terraform']['threshold'][SYSTEM_NAME]

    def run_terraform_scan():
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, repo_name)
        runner_filter = RunnerFilter()
        runner_registry = RunnerRegistry(banner, runner_filter, tf_runner())
        reports = runner_registry.run(root_folder=test_files_dir)
        assert len(reports) > 0

    benchmark(run_terraform_scan)
    assert benchmark.stats.stats.mean <= repo_threshold + (DEVIATION_PERCENT / 100.0) * repo_threshold


@pytest.mark.benchmark(
    group="cloudformation-performance-tests",
    disable_gc=True,
    min_time=0.1,
    max_time=0.5,
    min_rounds=5,
    timer=time.time,
    warmup=False
)
def test_cloudformation_performance(benchmark):
    repo_name = performance_configurations['cloudformation']['repo_name']
    repo_threshold = performance_configurations['cloudformation']['threshold'][SYSTEM_NAME]

    def run_cloudformation_scan():
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, repo_name)
        runner_filter = RunnerFilter()
        runner_registry = RunnerRegistry(banner, runner_filter, cfn_runner())
        reports = runner_registry.run(root_folder=test_files_dir)
        assert len(reports) > 0

    benchmark(run_cloudformation_scan)
    assert benchmark.stats.stats.mean <= repo_threshold + (DEVIATION_PERCENT / 100) * repo_threshold


@pytest.mark.benchmark(
    group="kubernetes-performance-tests",
    disable_gc=True,
    min_time=0.1,
    max_time=0.5,
    min_rounds=5,
    timer=time.time,
    warmup=False
)
def test_k8_performance(benchmark):
    repo_name = performance_configurations['kubernetes']['repo_name']
    repo_threshold = performance_configurations['kubernetes']['threshold'][SYSTEM_NAME]

    def run_kubernetes_scan():
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, repo_name)
        runner_filter = RunnerFilter()
        runner_registry = RunnerRegistry(banner, runner_filter, k8_runner())
        reports = runner_registry.run(root_folder=test_files_dir)
        assert len(reports) > 0

    benchmark(run_kubernetes_scan)
    assert benchmark.stats.stats.mean <= repo_threshold + (DEVIATION_PERCENT / 100) * repo_threshold



from __future__ import annotations

from checkov.github.repository_creation_is_limited import RepositoryCreationIsLimited


class GithubPrivateRepositoryCreationIsLimited(RepositoryCreationIsLimited):
    def __init__(self) -> None:
        name = "Ensure private repository creation is limited to specific members"
        id = "CKV_GITHUB_22"
        super().__init__(id=id, name=name)

    def get_evaluated_keys(self) -> list[str]:
        return ["members_can_create_private_repositories"]


check = GithubPrivateRepositoryCreationIsLimited()
